###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Date   : mclim.2020.05.12
###############################################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
CMP_IP      = '192.168.1.160'
UserDir     = '2020.07.30-Autolevel'
FSW_Rx      = True
freqArry    = [28e9]
pwrArry     = range(-50,10,2)                                       #Power Array
comment     = '-Autolevel'

###############################################################################
### Overhead
###############################################################################
from rssd.VSG.NR5G_K144     import VSG                              #pylint: disable=E0611,E0401
from rssd.VSA.NR5G_K144     import VSA                              #pylint: disable=E0611,E0401
from rssd.RCT.NR5G_KM601    import RCT                              #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO                           #pylint: disable=E0611,E0401
from rssd.RSI.time          import timer                            #pylint: disable=E0611,E0401

OFile = FileIO().makeFile(__file__)
TMR = timer()
SMW = VSG().jav_Open(SMW_IP,OFile)                                  #Create SMW Object
SMW.debug = 0
if FSW_Rx:
    FSW = VSA().jav_Open(FSW_IP,OFile)                              #Create FSW Object
    FSW.debug = 0
else:
    RCT().jav_Close()
    CMP = RCT().jav_Open(CMP_IP,OFile)                              #Create CMP Object

class dataClass():
    def __init__(self):
        self.Direction      = 'UL'
        self.CellID         = 1
        # self.FreqRng      = 'HIGH'
        self.ChBW           = 100
        self.TF             = 'ON'
        self.SubSp          = 120
        self.RB             = 60
        self.RBO            = 0
        self.Ch_RB          = 60
        self.Ch_RBO         = 0
        self.Mod            = 'QPSK'
        self.Rx             = ''
        self.pwr            = -100

def ReadSMW_Settings(NR5G):
    # NR5G.freq         = SMW.Get_Freq()
    NR5G.Direction      = SMW.Get_5GNR_Direction()
    NR5G.CellID         = SMW.Get_5GNR_BWP_CellID()
    SMW.subF            = 1
    # NR5G.FreqRng      = SMW.Get_5GNR_FreqRange()
    NR5G.ChBW           = SMW.Get_5GNR_ChannelBW()
    NR5G.TF             = SMW.Get_5GNR_TransPrecoding()
    NR5G.SubSp          = SMW.Get_5GNR_BWP_SubSpace()
    NR5G.RB             = SMW.Get_5GNR_BWP_ResBlock()
    NR5G.RBO            = SMW.Get_5GNR_BWP_ResBlockOffset()
    NR5G.Ch_RB          = SMW.Get_5GNR_BWP_Ch_ResBlock()
    NR5G.Ch_RBO         = SMW.Get_5GNR_BWP_Ch_ResBlockOffset()
    NR5G.Mod            = SMW.Get_5GNR_BWP_Ch_Modulation()
    return NR5G

def NR5G_Rx_Init():
    """Start 5GNR Measurement Channel"""
    if FSW_Rx:
        FSW.Init_5GNR()
        FSW.Set_5GNR_FrameCount('OFF')
    else:
        CMP.Init_5GNR()

def NR5G_Rx_Config(sSetting):
    if FSW_Rx:
        sSetting = sSetting.split('.nr5g')[0]
        FSW.Set_5GNR_AllocFile(f'C:\\R_S\\instr\\user\\Demo\\2020.07.30-Autolevel\\{sSetting}.allocation')
        FSW.Set_5GNR_FrameCount('OFF')
        FSW.Set_Trig1_Source('EXT')
        FSW.Set_SweepTime(3e-3)
        FSW.Set_5GNR_SubFrameCount(16)
        NR5G.Rx = FSW.Model + comment
    else:
        CMP.Init_5GNR()
        CMP.Set_5GNR_Path('P1.RRH.RF1')
        CMP.Set_Meas_Port('P1.RRH.RF1')
        CMP.Set_5GNR_BWP_Frame_Periodicity(2)
        CMP.Set_5GNR_BWP_Frame_SlotConfig(0,0,8,0)
        CMP.Set_5GNR_ChannelBW(100)
        CMP.Set_5GNR_CellID(NR5G.CellID)
        CMP.Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(2)
        # CMP.Set_5GNR_NumBWP()
        CMP.Set_5GNR_BWP_ResBlock(NR5G.RB, NR5G.RBO)
        CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTA BWP0, 1, 2, 1')                 #Config; AddPos; MaxLength
        CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTB BWP0, 1, 2, 1')                 #Config; AddPos; MaxLength
        CMP.Set_5GNR_TransPrecoding(NR5G.TF)
        CMP.Set_5GNR_PUSCH(NR5G.Ch_RB, NR5G.Ch_RBO, NR5G.Mod)
        CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:ADD 1, 2, 3, 0')          #Len; CDM; Pwr; Ant
        CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:SGEN CID, 0, 0')          #SeqType; DMRSID; N_SCID
        CMP.Set_5GNR_EVM_MeasOnExcept('ON')
        CMP.Set_5GNR_EVM_AvgCount(20)
        CMP.Set_5GNR_Trigger_Source('Free Run (Fast Sync)')
        NR5G.Rx = CMP.Model + comment

def NR5G_Rx_SetFreq(freq):
    if FSW_Rx:
        FSW.Set_Freq(freq)
        FSW.Set_5GNR_PhaseCompensate_Freq(freq)
    else:
        CMP.Set_5GNR_Freq(freq)
        CMP.Set_Meas_Freq(freq)
        CMP.Set_5GNR_PhaseCompensate_Freq(freq)

def NR5G_Rx_SetLevel(NR5G):
    if FSW_Rx:
        FSW.Set_SweepCont(1)
        FSW.Set_Autolevel()
        # FSW.Set_5GNR_AutoEVM()
    else:
        CF  = SMW.Get_CrestFactor()
        CMP.Init_Meas_Power()
        CMP.Set_Meas_UserMargin(CF)
        CMP.Set_Meas_Expected_Nom_Power(NR5G.pwr)
        CMP.Set_Meas_TriggerSource('IF Power')
        # CMP.Set_Meas_TriggerThreshold(NR5G.pwr)
        CMP.Set_Meas_TriggerThreshold(-40)
        CMP.Set_Meas_Pwr_MLength(100e-6)
        CMP.Set_Meas_RFBW(100e6)
        Pwr = CMP.Get_Meas_Power()
        CMP.Set_5GNR_ExpPwr(Pwr)
        CMP.Set_5GNR_UserMargin(CF)

def NR5G_Rx_Get_EVM():
    if FSW_Rx:
        FSW.Set_SweepCont(0)
        FSW.Set_InitImm()
        EVM = FSW.Get_5GNR_Params_EVM()
    else:
        EVM = CMP.Get_5GNR_Params_EVM()
    return EVM

###############################################################################
### Code Start
###############################################################################
LoopParam   = 'State,Model,SMW_Fre,SMW_Pwr'
WaveParam   = 'ChBW,SubSp,RB,Mod,TF'
AttnParam   = FSW.Get_Params_Amp(1) if FSW_Rx else CMP.Get_5GNR_Params_Amp(1)
EVMParam    = FSW.Get_5GNR_Params_EVM(1) if FSW_Rx else CMP.Get_5GNR_Params_EVM(1)
TimeParam   = 'AlTime,MeasTime,TotalTime,HoursLeft'
Header      = f'{LoopParam},{AttnParam},{EVMParam},{TimeParam}'
OFile.write(Header)

### Instr Init
NR5G        = dataClass()
SMW.Set_OS_Dir(UserDir)
SMW.Set_5GNR_BBState(1)
SMW.Set_RFPwr(-50)
SMW.Set_RFState(1)
saveArry = SMW.Get_OS_FileList('savrcltxt')
saveArry = SMW.Get_OS_FileList('nr5g')
NR5G_Rx_Init()

TMR.numTest = len(saveArry) * len(freqArry) * len(pwrArry)
TMR.suite_start()
for saveState in saveArry:
    SMW.Set_5GNR_Setting_Load(f'{UserDir}/{saveState}')
    ReadSMW_Settings(NR5G)
    NR5G_Rx_Config(saveState)
    for freq in freqArry:
        SMW.Set_Freq(freq)
        SMW.Set_5GNR_PhaseCompensate_Freq(freq)
        NR5G_Rx_SetFreq(freq)
        for pwr in pwrArry:
            TMR.start()
            SMW.Set_RFPwr(pwr)
            # SMW.Set_OptimizeAll()
            NR5G.pwr = pwr
            NR5G_Rx_SetLevel(NR5G)
            TMR.tick()
            EVM = NR5G_Rx_Get_EVM()
            TMR.tick()

            ### Log Data
            LoopParam   = f'{saveState},{NR5G.Rx},{freq},{pwr:3d}'
            NR5GParam   = f'{NR5G.ChBW},{NR5G.RB},{NR5G.SubSp},{NR5G.Mod},{NR5G.TF}'
            AttnParam   = FSW.Get_Params_Amp() if FSW_Rx else CMP.Get_5GNR_Params_Amp(0)
            TimeParam   = TMR.Get_Params_Time()
            OutStr      = f'{LoopParam},{AttnParam},{EVM},{TimeParam}'
            OFile.write(OutStr)
SMW.Set_RFPwr(-100)



from datetime           import datetime
from rssd.VSE.Common    import VSE
from rssd.FileIO        import FileIO

OFile = FileIO().makeFile(__file__)
FSW = VSE().jav_Open('127.0.0.1',OFile)  #Create FSW Object

################################################################################
### Code Start
################################################################################
FSW.Init_IQ()                       #FSW IQ Channel
FSW.Set_DisplayUpdate("OFF")
FSW.Set_SweepCont(0)

OFile.write('Fs,CapTime,Iter,CmdTime')
for Fs in [491.52e6, 1351.68e6]:
    for measTime in [10e-3, 20e-3]:
        print(f'Fs:{Fs} Meas:{measTime}')
        FSW.Set_IQ_SamplingRate(Fs)
        FSW.Set_SweepTime(measTime)
        tick = datetime.now()
        FSW.Set_InitImm()
        FSW.Get_IQ_Data()
        d = datetime.now() - tick
        OutStr = f'{Fs/1e6},{measTime},{0:2d},{d.seconds:03d}.{d.microseconds:06d}'
        OFile.write(OutStr)

FSW.jav_Close()



'''RSSD VISA Bus class'''
import os
import pyvisa
import logging
from rssd.RSI.time      import timer
from rssd.bus.bus       import bus

class jaVisa(bus):
    """Rohde & Schwarz VISA Class"""
    def __init__(self):
        self.EOL        = '\n'              # \n or \r\n
        self.VISA       = ''                # '@py' for pyvisa-py
        self.debug      = 1
        logging.basicConfig(level=logging.INFO, \
                            filename=os.path.splitext(__file__)[0] + '.log', filemode='w', \
                            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    def close(self):
        """Close VISA Session"""
        try:
            self.K2.close()
            self.rm.close()
        except:
            logging.info(f'jaV-Close : Could not close')

    def open(self, resourceID, param=0):            #pylint: disable=unused-argument
        """
        Open VISA object w/ VISA String.

        Examples:
            TCPIP0::<IP_Address>::inst0::INSTR
            TCPIP0::<IP_Address>::hislip0::INSTR
            TCPIP0::<IP_Address>::5025::SOCKET
            GPIB::<Addr>::INSTR
            ASRL1::INSTR
        """
        TMR = timer()
        TMR.start()
        self.rm = pyvisa.ResourceManager(self.VISA)                     # Create Resource Manager
        self.ResID = resourceID
        TMR.tick()
        try:
            # self.K2 = rm.open_resource(resourceID, open_timeout=100)  # Create Visa Obj
            self.K2 = self.rm.open_resource(self.ResID)                 # Create Visa Obj
            self.K2.timeout = 5000                                      # Timeout, millisec
            self.K2.write_termination = self.EOL
            self.K2.read_termination  = self.EOL
        except:
            logging.info(f'jaV_OpnErr: {self.ResID}')
            self.K2 = 'NoVISA'
        TMR.tick()
        asdf = TMR.Get_Params_Time()
        return self

    def query(self, SCPIstr):
        rdStr = '<notRead>'
        try:
            rdStr = self.K2.query(SCPIstr)
        except AttributeError:                                          # K2 = string
            logging.info(f'jaV_RdErr : {self.ResID}-->{SCPIstr}')
        return rdStr

    def read_raw(self):
        return self.K2.read_raw()

    def reslist(self):
        try:
            rmList = self.rm.list_resources()                           #List VISA Resources
        except:
            rmList =["No VISA"]
        return rmList

    def timeout(self, seconds):
        self.K2.timeout = seconds * 1000

    def write(self, SCPIstr):
        try:
            self.K2.write(SCPIstr)
        except AttributeError:                                          # K2 = string
            logging.info(f'jaV_WrtErr: {self.ResID}-->{SCPIstr}')

    def write_raw(self, SCPIstr):
        self.K2.write_raw(SCPIstr)

if __name__ == "__main__":
    ipaddress   = '192.168.58.15'
    RS = jaVisa().open(f'TCPIP::{ipaddress}::hislip0::INSTR')           # hislip
    RS = jaVisa().open(f'TCPIP::{ipaddress}::instr0::INSTR')            # VXI11
    RS = jaVisa().open(f'TCPIP::{ipaddress}::5025::SOCKET')             # Socket
    print(RS.query('*IDN?'))

    RS.close()



""" Timing SCPI Commands Example"""
import  timeit
from rssd.yaVISA_socket     import jaVisa
from rssd.FileIO            import FileIO

OFile = FileIO().makeFile(__file__)
instr = jaVisa().jav_Open('192.168.58.109', OFile)                 #Create Object
instr.K2.settimeout(30)

###############################################################################
### Code Start
###############################################################################
OFile.write('Iter,CmdTime,Response')

ALTime = []
for i in range(1):
    rdStr = ''
    tick = timeit.default_timer()

    ### <\thing we are timing>
    instr.query('INIT:IMM;*OPC?')
    rdStr = instr.query('FETC:SUMM:EVM:ALL?')
    ### <\thing we are timing>

    ALTime.append(timeit.default_timer() - tick)
    OutStr = f'{i},{ALTime[i]:.6f},{rdStr}'
    OFile.write(OutStr)

print(f'Avg time  : {sum(ALTime) / float(len(ALTime))} secs')

instr.jav_Close()



"""
Vector Signal Analyzer 802.11 WLAN functions
#  | Standrd  | Modu | Freqncy | Mode | PPDU | BW  | MIMO | BitRate  | SCS,KHz | Sym,uSec |
#  | -------- | ---- | ------- | ---- | ---- | --- | ---- | -------- | ------- | -------- |
#  | 802.11b  | DSSS | 2.4     | Legy | CCK  | 20  | SISO | 11Mbps   | n/a     | n/q      | 
#  | 802.11a  | OFDM | 5.4     | Legy |      | 20  | SISO | 54Mbps   | 312.5   | 3.2 μs   |
#  | 802.11g  | both | 2.4     | Legy | L    | 20  | SISO | 54Mbps   | 312.5   | 3.2 μs   |
#  | 802.11n  | OFDM | 2.4 5.4 | GrnF | HT   | 40  | MIMO | 300Mbps  | 312.5   | 3.2 μs   |
#  | 802.11ac | OFDM | 5.4     | MixM | VHT  | 160 | MIMO | 1750Mbps | 312.5   | 3.2 μs   |
#  | 802.11ax | OFDM | 2.4 5.4 | MixM | HE   | 160 | MIMO | 1201Mbps | 78.125  | 12.8 μs  |
#  | 802.11be | OFDM | 2.4 5.4 | MixM | EHT  | 320 | MIMO | 11Gbps   | 78.125  | 12.8 μs  | 
"""
from rssd.VSA.Common import VSA        # pylint: disable=E0611,E0401

class VSA(VSA):                        # pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Analyzer 802.11 Object """
    def __init__(self):
        super(VSA, self).__init__()
        self.WLAN_Std  = 'N'
        self.WLAN_ChBW = 100       #MHz
        self.WLAN_MCS  = 1

    #####################################################################
    ### VSA Query
    #####################################################################
    def Get_ACLR(self):
        ACLR = self.queryFloatArry(':CALC:MARK:FUNC:POW:RES? MCAC')
        return ACLR

    def Get_Params_WLAN_EVM(self,header=0):
        """Retrieve Parameters for test logs"""
        if header != 1:
            Burst = self.Get_WLAN_BurstCount()
            Power = self.Get_WLAN_PPDUPwr()
            EVM   = self.Get_WLAN_EVM()
            outStr = f"{Burst},{Power:6.3f},{EVM:.2f}"
        else:
            outStr = 'BurstCnt,PPDUPwr,EVM'
        return outStr

    def Get_WLAN_BurstCount(self):
        rdStr = self.query(f'FETC:BURS:COUN:ALL?')
        rdStr = self.query(f'FETC:BURS:COUN?')
        return rdStr

    def Get_WLAN_PPDUPwr(self):
        try:
            rdStr = self.queryFloatArry(f'FETC:BURS:RMS:AVER?')[1]
        except:
            rdStr = -9999
        return rdStr

    def Get_WLAN_ChBW(self):
        #rdStr = self.query(f':SENS:BAND:CHAN:AUTO:TYPE?')
        rdStr = '<TBD>'
        return rdStr

    def Get_WLAN_EVM(self):
        rdStr = self.queryFloat('FETC:BURS:EVM:ALL:AVER?')
        return rdStr

    def Get_WLAN_Modulation(self):
        # FSW-->Demod -->
        rdStr = self.query(f':SENS:DEM:FORM:BAN?')      #AC
        return rdStr

    def Get_WLAN_MCS(self):
        # For n & ac
        rdStr = 'MCS' + self.query(f'SENS:DEM:FORM:MCS?')
        return rdStr

    def Get_WLAN_PPDU(self):
        std = self.Get_WLAN_Standard()
        if std == 'N':
            rdStr = self.query(':TRAC:DATA? TRACE1')
        elif std == 'AC':
            rdStr = self.query(':TRAC:DATA? TRACE1')
        elif std == 'AX':
            rdStr = self.query('FETCh:SFIeld:ALL?')
        else:
            rdStr = '<TBD>'
        return rdStr

    def Get_WLAN_SEM(self):
        rdStr = self.query(f':CALC1:LIM:FAIL?')
        return rdStr

    def Get_WLAN_Standard(self):
        #0:A 1:B 2/3:J 4:G 6:N 7:N-MIMO 8:AC 9:P 10:AX
        rdStr = self.queryInt(f':CONF:STAN?')
        if rdStr == 0:
            rdStr = 'A'
        elif rdStr == 1:
            rdStr = 'B'
        elif rdStr == 4:
            rdStr = 'G'
        elif rdStr == 6:
            rdStr = 'N'
        elif rdStr == 8:
            rdStr = 'AC'
        elif rdStr == 10:
            rdStr = 'AX'
        return rdStr

    #####################################################################
    ### Init WLAN
    #####################################################################
    def Init_WLAN(self):
        self.Set_Channel('WLAN')
        self.write(':SENS:DEM:FORM:BCON:AUTO 1')            #Auto PPDU Demod

    def Init_WLAN_ACLR(self):
        self.Set_Channel('WLAN')
        self.write(':CONF:BURS:SPEC:ACPR:IMM')              #ACLR

    def Init_WLAN_EVM(self):
        self.Set_Channel('WLAN')
        self.write(':CONF:BURS:IQ:IMM')                     #EVM
        self.write(':SENS:DEM:FORM:BCON:AUTO 1')            #Auto PPDU Demod

    def Init_WLAN_SEM(self):
        self.Set_Channel('WLAN')
        self.write(':CONF:BURS:SPEC:MASK:IMM')              #SEM

    #####################################################################
    ### VSA Settings
    #####################################################################
    def Set_WLAN_AnalysisMode(self):
        self.write(':SENS:DEM:FORM:BCON:AUTO 1')

    def Set_WLAN_Autolvl(self):
        """ Supports B40; B80; B2001 not supported."""
        #self.query('ADJ:LEV;*OPC?')
        self.write(':CONF:POW:AUTO ONCE;*WAI')      #FSVA

    def Set_WLAN_CaptureTime(self, swpTime):
        self.Set_SweepTime(swpTime)

    def Set_WLAN_ChBW(self,iBW):
        # iBW of 0 sets VSA to auto detect
        if iBW == 0:  # Auto
            self.write(f':SENS:BAND:CHAN:AUTO:TYPE ALL')
        else:
            self.write(f':SENS:BAND:CHAN:AUTO:TYPE MB{iBW:02d}')

    def Set_WLAN_MIMO_Streams(self, iAnt):
        self.write(f'CONF:WLAN:DUTC TX{iAnt}')

    def Set_WLAN_Modulation(self,iMod):
        self.write(f':CONF:LTE:UL:SUBF0:ALL:MOD {iMod}')

    def Set_WLAN_MCS(self,iMCS):
        # iMCS of 0 sets VSA to auto detect
        if iMCS == 0:
            self.write(f'SENS:DEM:FORM:MCS:MODE ALL')
        else:
            self.write(f'SENS:DEM:FORM:MCS:MODE MEAS')
            self.write(f'SENS:DEM:FORM:MCS {iMCS}')

    def Set_WLAN_Standard(self,sStd):
        """AC N AX"""
        #0:A 1:B 2/3:J 4:G 6:N 7:N-MIMO 8:AC 9:P 10:AX
        sStd.upper()
        if sStd == 'A':
            self.write(f':CONF:STAN 0')
        elif sStd == 'B':
            self.write(f':CONF:STAN 1')
        elif sStd == 'G':
            self.write(f':CONF:STAN 4')
        elif sStd == 'N':
            self.write(f':CONF:STAN 6')
        elif sStd == 'AC':
            self.write(f':CONF:STAN 8')
        elif sStd == 'AX':
            self.write(f':CONF:STAN 10')
        else:
            print(f'Set_WLAN_Standard {sStd} not supported')


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA()
    FSW.jav_Open("192.168.1.109")
    print(FSW.query(':TRAC:DATA? TRACE1'))
    #print(FSW.Get_WLAN_EVMParams())
    #FSW.Set_WLAN_Autolvl()
    FSW.jav_Close()



"""rssd.test.examples"""

from __future__ import print_function
import unittest
import rssd.test.yaVISA
import rssd.test.FileIO

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        from importlib import reload
        reload(rssd.test.yaVISA)
        reload(rssd.test.FileIO)
        print("",end="")

    def tearDown(self):                             #Run after each test
        pass

###############################################################################
### <Test>
###############################################################################
    def test_EX_helloworld(self):
        import rssd.examples.AAA_CommandTime as example

    # def test_EX_CMW_GPRF_Loopback(self):
    #     import rssd.examples.CMW_GPRF_Loopback as example
    #     self.assertTrue(1)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # unittest.main()     #Run w/o test names
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""
Rohde & Scharz Instrument Memory/File Functions
 _____  _____   ____ _______ ____ _________     _______  ______
|  __ ||  __ | / __ |__   __/ __ |__   __| |   / /  __ ||  ____|
| |__) | |__) | |  | | | | | |  | | | |   | |_/ /| |__) | |__
|  ___/|  _  /| |  | | | | | |  | | | |    |   / |  ___/|  __|
| |    | | | || |__| | | | | |__| | | |     | |  | |    | |____
|_|    |_|  |_||____/  |_|  |____/  |_|     |_|  |_|    |______|
                        _            _           _
                       | |          | |         | |
            _   _ _ __ | |_ ___  ___| |_ ___  __| |
           | | | | '_ || __/ _ |/ __| __/ _ |/ _` |
           | |_| | | | | ||  __/|__ | ||  __/ (_| |
            |__,_|_| |_||__|___||___/|__|___||__,_|
"""
import timeit
from rssd.yaVISA import jaVisa

class RSI(jaVisa):
    """ Rohde & Schwarz Instrument Memory Object """
    def __init__(self):
        super(RSI, self).__init__()
        self.Model = "AAA"

    ##########################################################################
    ### AAA Get Methods
    ##########################################################################
    def Get_dir(self):
        rdStr = self.query(f':MMEM:CDIR?')
        return rdStr

    def Get_dirInfo(self):
        # FSW/FSVA uses ':MMEM:CAT:LONG?'
        # SMW/ZVA  uses ':MMEM:CAT?'
        dirInfo = self.query(':MMEM:CAT:LONG?')
        if '<notRead>' in dirInfo:
            dirInfo = self.query(':MMEM:CAT?')
        dirInfo = dirInfo.strip().replace('"','').split(',')
        print(dirInfo,'\n')
        if len(dirInfo) > 2:
            self.usedSpace = dirInfo.pop(0)
            self.freeSpace = dirInfo.pop(0)
            self.dir       = []
            self.files     = {}
            for i in range(0, len(dirInfo), 3):
                if dirInfo[i+1] == 'DIR':
                    self.dir.append(dirInfo[i])                 #Add directories to self.dir
                else:
                    self.files[dirInfo[i]]= int(dirInfo[i+2])   #Add file to self.files
        return dirInfo

    def Get_File(self,filename):                                #MMM
        """ Return File Data
            Args:
                param1: filename
        """
        self.Get_dirInfo()
        self.write(f':MMEM:DATA? {filename}')
        size = self.files[filename]
        rdStr = self.jav_read_raw()
        return rdStr

    ###########################################################################
    ### AAA Set Methods
    ###########################################################################
    def Set_Copy(self,fromLoc, toLoc):
        """ Copy files from <source> to <destination>
            Ex: Set_Copy('temp.txt','temp2.txt')
        """
        self.write(f'MMEM:COPY "{fromLoc}","{toLoc}"')

    def Set_Dir(self, pathy):
        """ Set_Dir('C:\\R_S\\Instr')
            Set_Dir('C:\\R_S\\Instr\\')
        """
        self.write(f'MMEM:CDIR "{pathy}"')

    def Set_FileCreate(self,filename):
        """Set_FileCreate('temp.txt')"""
        self.Set_Filename(filename)
        self.Set_FileWrite(filename)

    def Set_FileDel(self,filename):
        self.write(f"MMEM:DEL '{filename}'")

    def Set_Filename(self,filename):
        self.write(f'MMEM:NAME "{filename}"')

    def Set_FileWrite(self,filename):
        self.write(f'MMEM:DATA "{filename}",#10')

    def Set_Move(self,fromLoc, toLoc):
        self.write(f'MMEM:MOVE "{fromLoc}","{toLoc}"')

###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    RSI = RSI().jav_Open("192.168.1.108")
    tick = timeit.default_timer()
    RSI.Set_FileCreate()

    #####################################
    ### Begin Timer
    #####################################
    RSI.Set_FileCreate('temp.txt')
    # RSI.Get_dirInfo
    # RSI.Set_FileDel('temp.txt')
    # RSI.Set_Copy('temp.txt','temp2.txt')
    print(RSI.Get_dir())
    #####################################
    ### End Timer
    #####################################
    a = timeit.default_timer() - tick
    print(f'Time: {a:.6f}')
    RSI.jav_ClrErr()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Purpose: Timer Object
### Author : Martin C Lim
###############################################################################
import timeit
import time

class timer():                                                       #pylint: disable=E0102
    def __init__(self):
        self.timeStart  = 0
        self.numTest    = 100
        self.curTest    = 1
        self.ticks      = []
        self.n          = 0

    def start(self):
        self.ticks = []
        self.tick()

    def tick(self):
        self.ticks.append(timeit.default_timer())

    def deltaTimes(self):
        deltaTimes  = []
        maxIndx     = len(self.ticks)-1
        for i,tick in enumerate(self.ticks):
            if (i < maxIndx):
                deltaTimes.append(self.ticks[i+1] - self.ticks[i])
        deltaTimes.append(self.ticks[maxIndx] - self.ticks[0])

        ### Calculate time left
        remTest = self.numTest - self.curTest
        deltaTime = (timeit.default_timer() - self.timeStart) / self.curTest
        timeLeft = deltaTime * (remTest) / (60 * 60)
        deltaTimes.append(timeLeft)     # in hours
        self.curTest += 1
        return deltaTimes

    def Get_Params_Time(self, header=0):
        if header != 1:
            outStr = ""
            for each in self.deltaTimes():
                outStr += f'{each:2,.3f},'
            outStr = outStr[0:-1]
        else:
            outStr = 'deltattime1,totalTime,Hrs2Done'
        return outStr

    def suite_start(self):
        self.timeStart = timeit.default_timer()


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    TMR = timer()
    print(TMR.Get_Params_Time(1))
    TMR.start()
    time.sleep(1.2)
    TMR.tick()
    time.sleep(0.6)
    TMR.tick()
    print(TMR.Get_Params_Time())



'''RSSD instrument object'''
import os
import time
import logging
import rssd.FileIO
from rssd.bus.jaSocket      import jaSocket
from rssd.bus.jaVISA        import jaVisa
from rssd.bus.test          import jaTest

class instr(object):
    '''Rohde & Schwarz Instrument Class'''
    def __init__(self):
        self.dataIDN   = ""         # Raw IDN String
        self.Make      = ""         # IDN Make
        self.Model     = ""         # IDN Model
        self.Device    = ""         # IDN Device
        self.Version   = ""         # IDN Version
        self.EOL       = '\r\n'
        self.dLastErr  = ''         # Last error
        self.bus       = 'Nobus'    # bus object
        self.connected = 0
        logging.basicConfig(level=logging.INFO, \
                            filename=os.path.splitext(__file__)[0] + '.log', filemode='a', \
                            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    def close(self):
        '''Close bus Session'''
        try:
            errList = self.SCPI_clrErr()
            self.bus.close()
            return errList
        except:
            pass

    def delay(self,sec):
        '''delay in Sec'''
        time.sleep(sec)

    def open(self, address, type = 'socket', param = 5025):         # pylint: disable=redefined-builtin
        '''Open bus Sesion.  Return bus object'''
        if type == 'socket':
            self.bus = jaSocket().open(address, param)
        elif type == 'visa-socket':
            self.bus = jaVisa().open(f'TCPIP0::{address}::{param}::SOCKET')
        elif type == 'vxi11':
            self.bus = jaVisa().open(f'TCPIP0::{address}::instr0::INSTR')
        elif type == 'hislip':
            self.bus = jaVisa().open(f'TCPIP0::{address}::hislip0::INSTR')
        elif type == 'test':
            self.bus = jaTest().open('test')
            self.SCPI_error      = jaTest.SCPI_error
        self.SCPI_IDN()
        self.SCPI_file_write(self.dataIDN)
        self.SCPI_clrErr()
        return self

    def query(self,cmd):
        read ="<notRead>"
        try:
            if self.dataIDN != "":
                read = self.bus.query(cmd).strip()                   # Write if connected
        except:
            logging.error(f'SCPI_RdErr : {self.Model}-->{cmd}')
        self.SCPI_file_write(f'{self.Model},{cmd},{read}')
        return read

    def queryFloat(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [float(i) for i in strArry][0]
        except:
            return -9999.9999

    def queryFloatArry(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [float(i) for i in strArry]
        except:
            return [-9999.9999, -888.888, -777.777, -666.666, -555.555, -444.444]

    def queryInt(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return int([float(i) for i in strArry][0])
            # Float for scientific 'e' notation
        except:
            return -9999

    def queryIntArry(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [int(i) for i in strArry]
        except:
            return [-9999,-8888,-7777]

    def SCPI_clear(self):
        '''Clear Errors'''
        self.bus.clear()

    def SCPI_clrErr(self):
        '''Read all SYST:ERR messages'''
        ErrList = []
        try:                                                        # Instr supports SYST:ERR?
            while True:
                RdStr = self.SCPI_error()
                if RdStr[0] == "<notRead>" : break                  # No readstring
                if RdStr[0] == "0" : break                          # Read 0 error:R&S
                if RdStr[0] == "+0": break                          # Read 0 error:Other
                if RdStr[1] == ' 0' and (RdStr[3] == ' 0') : break  # ATS1800
                self.dLastErr = RdStr
                ErrList.append(RdStr)
                logging.error(f'SCPI_ClrErr: {self.Model}-->{RdStr}')
        except:  #Instrument does not support SYST:ERR?
            logging.error('SCPI_ClrErr: {self.Model}-->SYST:ERR not Supported')
        return ErrList

    def SCPI_error(self):
        '''Read SYST:ERR?'''
        RdStr = self.query("SYST:ERR?").strip().split(',')
        return RdStr

    def SCPI_file_write(self, outstr):
        '''Write SCPI to file if f object exists'''
        logging.info(outstr.strip())

    def SCPI_IDN(self):
        '''query *IDN?  Assign data to properties'''
        self.dataIDN = "Temp"                                       # Temp for self.query
        self.dataIDN = self.query("*IDN?").strip()
        if self.dataIDN != "<notRead>":                             # Data Returned?
            IDNStr = self.dataIDN.split(',')
            try:
                self.Make       = IDNStr[0]
                self.Model      = IDNStr[1]
                self.Device     = IDNStr[2]
                self.Version    = IDNStr[3]
            except:
                pass
        else:
            self.dataIDN = ""                                       # Reset if not read
        return self.dataIDN

    def SCPI_logscpi(self):
        self.f = rssd.FileIO()                                      # pylint:disable=E1101
        DataFile = self.f.init("yaVISA")                            # pylint:disable=W0612

    def SCPI_read_OPC(self, InCMD):
        ''' Wait based on *OPC '''
        start_time = time.time()
        self.write("*ESE 1")                                        # Event Status Enable
        self.write("*SRE 32")                                       # ServiceReqEnable-Bit5:Std Event
        self.query('*ESR?')                                         # Clear ESR flag
        self.write(f'{InCMD};*OPC')                                 # Initiate Read.  *OPC will trigger ESR
        while (self.queryInt("*STB?") & 32) != 32:                  # Loop until done
            time.sleep(0.1)
            delta = (time.time() - start_time)
            if delta > 300:
                logging.error('SCP_OPCWai: timeout')
                break
        logging.error(f'SCP_OPCWai: {delta:0.2f}sec')
        self.SCPI_clrErr()
        return delta

    def SCPI_read_raw(self):
        '''read raw data from bus'''
        return self.bus.read_raw()

    def SCPI_reset(self):
        '''reset instrument'''
        self.write("*RST;*CLS;*WAI")

    def SCPI_read_wait(self, InCMD):
        '''Brute Force Wait and check *OPC? '''
        start_time = time.time()
        self.write(InCMD)                                           # Initiate busand
        read = "0"
        while (int(read) & 1) != 1:                                 # Loop until done
            try:
                read = self.queryInt("*OPC?")                       # See if we can get *OPC?
            except:
                pass
            time.sleep(2)
            delta = (time.time() - start_time)
            if delta > 300:
                logging.error('SCPI_Wai   : timeout')
                break
        logging.error(f'SCPI_Wai   : {delta:0.2f}sec')
        return delta

    def timeout(self, seconds):
        self.bus.timeout(seconds)

    def write(self,cmd):
        try:
            if self.dataIDN != "": self.bus.write(cmd)               # Write if connected
        except:
            logging.error(f'SCPI_WrtErr : {self.Model}-->{cmd}')
        self.SCPI_file_write(f'{self.Model},{cmd}')

    def write_raw(self,SCPI):
        self.bus.write_raw(SCPI)

    def write_scpilist(self,SCPIList):
        '''Send SCPI list & Query if "?" '''
        ### Collect read results into a list for return.
        OutList = []
        for cmd in SCPIList:
            if cmd.find('?') == -1:
                self.write(cmd)
            else:
                ReadStr = self.query(cmd)
                OutList.append(ReadStr)
        return OutList

if __name__ == "__main__":
    RS = instr().open('192.168.58.109')                                 # Default HiSlip
    RS.SCPI_read_OPC(':INIT:IMM')
    RS.close()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
#pylint: disable=E0611,E0401
###############################################################################
SMW_IP      = '192.168.58.114'
freqArry    = [28e9]

###############################################################################
### Overhead
###############################################################################
from rssd.VSG.NR5G_K144     import VSG

SMW = VSG().jav_Open(SMW_IP)                          #Create SMW Object
SMW.debug = 0

###############################################################################
### Code Start
###############################################################################
SMW.Set_Freq(3.5e9)
SMW.Set_5GNR_BBState('OFF')
SMW.Set_5GNR_Direction('UL')
# SMW.Set_5GNR_BWP_CellID(1024)
SMW.write(':SOUR1:BB:NR5G:UBWP:USER0:UEID 1024')
SMW.Set_5GNR_FreqRange('LOW')
SMW.Set_5GNR_ChannelBW(100)
SMW.Set_5GNR_BWP_Users(50)

for ch in range(0,44):
    ## Frame Configuration
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:NALL 1')                   # Add User
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:CONT PUCC')           # Select PUCCH
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:FMT F0')              # Format
    # SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:RBN 1')             # Number of RB
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:RBOF {ch}')           # RB Offset
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:SYMN 2')              # Number of Symbol
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:SYM 12')              # Symbol offset
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:REP SLOT')            # Repitition

    ## PUCCH Setting
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:ISFH 1')         # Intra Slot Freq Hop
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:HOP {ch * 2}')   # Hop ID
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:SHOP {272-ch}')  # 2nd Hop PBR
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:FS:CYCS 6')   # Initial Cyclic Shift

RBSize = 1
for ch in range(44,50):
    ## Frame Configuration
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:NALL 1')                   # Add User
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:CONT PUCC')           # Select PUCCH
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:FMT F2')              # Format
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:RBN {RBSize}')        # Number of RB
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:RBOF {ch}')           # RB Offset
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:SYMN 2')              # Number of Symbol
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:SYM 0')               # Symbol offset
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:REP SLOT')            # Repitition

    ## PUCCH Setting
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:ISFH 1')         # Intra Slot Freq Hop
    # SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:HOP {ch * 2}')   # Hop ID
    SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:SHOP {272-ch}')  # 2nd Hop PBR
    # SMW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:PUCC:FS:CYCS 6')   # Initial Cyclic Shift

SMW.Set_5GNR_BBState('ON')



"""FSW/SMW 5G NR Demo"""
#pylint: disable=E0611,E0401
from rssd.VSG.NR5G_K144 import VSG
from rssd.VSA.NR5G_K144 import VSA

class VST(object):
    """ Rohde & Schwarz Vector Signal Transciever 5GNR Object """
    def __init__(self):
        self.Freq      = 19e9
        self.SWM_Out   = 0
        self.NR_Dir    = 'UL'
        self.NR_Deploy = 'HIGH'     #LOW|MIDD|HIGH
        self.NR_PhaseC = 'OFF'      #ON|OFF
        self.NR_ChBW   = 100        #MHz
        self.NR_SubSp  = 120        #kHz
        self.NR_RB     = 66         #RB
        self.NR_RBO    = 0          #RB Offset
        self.NR_Mod    = 'QAM64'    #QPSK; QAM16; QAM64; QAM256; PITB
        self.NR_CC     = 1
        self.NR_TF     = 'OFF'

    def Get_5GNR_All(self):                     #pylint: disable=R0915
        DMRS = 1
        PTRS = 1
        self.SMW.cc     = self.NR_CC - 1
        self.FSW.cc     = self.NR_CC

        odata =  [[] for i in range(3)]
        odata[0].append("[[Parameter]]  ")
        odata[0].append("Center Freq    ")
        odata[0].append("Direction      ")
        odata[0].append("FreqRange      ")
        odata[0].append("RefA,MHz       ")
        odata[0].append("Ch BW          ")
        odata[0].append("TransPrecoding ")
        odata[0].append("Phase Compensat")
        odata[0].append("Phase Comp Freq")
        odata[0].append("====SS/PBCH====")
        odata[0].append("SubSpacing     ")
        odata[0].append("===User/BWP====")
        odata[0].append("SubSpacing     ")
        odata[0].append("Num BWP        ")
        odata[0].append("BWP_RB         ")
        odata[0].append("BWP_RBoff      ")
        odata[0].append("====Channel====")
        odata[0].append("User_BWP_Mod   ")
        odata[0].append("User_BWP_RB    ")
        odata[0].append("User_BWP_RBOff ")
        odata[0].append("User_BWP_SymNum")
        odata[0].append("User_BWP_SymOff")
        odata[0].append("User_BWP_Cntr  ")
        if DMRS:
            odata[0].append("=====DMRS======")
            odata[0].append("DMRS Mapping   ")
            odata[0].append("DMRS FirstSym  ")
            odata[0].append("DMRS Config    ")
            odata[0].append("DMRS Add Positn")
            odata[0].append("DMRS Length    ")
            odata[0].append("DMRS SeqGenMeth")
            odata[0].append("DMRS SeqGenSeed")
            odata[0].append("DMRS Rel Power ")
        if PTRS:
            odata[0].append("=====PTRS======")
            odata[0].append("PTRS State     ")
            odata[0].append("Time L_PTRS    ")
            odata[0].append("Freq K_PTRS    ")
            odata[0].append("PTRS Rel Power ")
            odata[0].append("UL-PTRS-RE-offs")

        try:
            # self.SMW.Set_5GNR_Parameters(self.NR_Dir)
            odata[1].append("[-SMW-]")
            odata[1].append(self.SMW.Get_5GNR_CC_Freq())
            odata[1].append(self.SMW.Get_5GNR_Direction())
            odata[1].append(self.SMW.Get_5GNR_FreqRange())
            odata[1].append(self.SMW.Get_5GNR_RefA()/1e6)
            odata[1].append(self.SMW.Get_5GNR_ChannelBW())
            odata[1].append(self.SMW.Get_5GNR_TransPrecoding())
            odata[1].append(self.SMW.Get_5GNR_PhaseCompensate())
            odata[1].append(self.SMW.Get_5GNR_PhaseCompensate_Freq())
            odata[1].append("=SSB==")
            odata[1].append(self.SMW.Get_5GNR_SSB_SubSpace())
            odata[1].append("=User=")
            odata[1].append(self.SMW.Get_5GNR_BWP_SubSpace())
            odata[1].append(self.SMW.Get_5GNR_BWP_Count())
            odata[1].append(self.SMW.Get_5GNR_BWP_ResBlock())
            odata[1].append(self.SMW.Get_5GNR_BWP_ResBlockOffset())
            odata[1].append("==Ch==")
            odata[1].append(self.SMW.Get_5GNR_BWP_Ch_Modulation())
            odata[1].append(self.SMW.Get_5GNR_BWP_Ch_ResBlock())
            odata[1].append(self.SMW.Get_5GNR_BWP_Ch_ResBlockOffset())
            odata[1].append(self.SMW.Get_5GNR_BWP_Ch_SymbNum())
            odata[1].append(self.SMW.Get_5GNR_BWP_Ch_SymbOff())
            odata[1].append(self.SMW.Get_5GNR_BWP_Center()/1e6)
            if DMRS:
                odata[1].append("=DMRS=")
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_Mapping())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_1stDMRSSym())

                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_Config())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_AddPosition())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_MSymbLen())

                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_SeqGenSeed())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_DMRS_RelPwr())
            if PTRS:
                odata[1].append("=PTRS=")
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_PTRS_State())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_PTRS_L())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_PTRS_K())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_PTRS_Pow())
                odata[1].append(self.SMW.Get_5GNR_BWP_Ch_PTRS_RE_Offset())
        except:
            pass

        try:
            self.FSW.Init_5GNR()
            odata[2].append("[-FSW-]")
            odata[2].append(self.FSW.Get_5GNR_CC_Freq())
            odata[2].append(self.FSW.Get_5GNR_Direction())
            odata[2].append(self.FSW.Get_5GNR_FreqRange())
            odata[2].append(self.FSW.Get_5GNR_RefA()/1e6)
            odata[2].append(self.FSW.Get_5GNR_ChannelBW())
            odata[2].append(self.FSW.Get_5GNR_TransPrecoding())
            odata[2].append(self.FSW.Get_5GNR_PhaseCompensate())
            odata[2].append(self.FSW.Get_5GNR_PhaseCompensate_Freq())
            odata[2].append("=SSB==")
            odata[2].append(self.FSW.Get_5GNR_SSB_SubSpace())
            odata[2].append("=User=")
            odata[2].append(self.FSW.Get_5GNR_BWP_SubSpace())
            odata[2].append(self.FSW.Get_5GNR_BWP_Count())
            odata[2].append(self.FSW.Get_5GNR_BWP_ResBlock())
            odata[2].append(self.FSW.Get_5GNR_BWP_ResBlockOffset())
            odata[2].append("==Ch==")
            odata[2].append(self.FSW.Get_5GNR_BWP_Ch_Modulation())
            odata[2].append(self.FSW.Get_5GNR_BWP_Ch_ResBlock())
            odata[2].append(self.FSW.Get_5GNR_BWP_Ch_ResBlockOffset())
            odata[2].append(self.FSW.Get_5GNR_BWP_Ch_SymbNum())
            odata[2].append(self.FSW.Get_5GNR_BWP_Ch_SymbOff())
            odata[2].append(self.FSW.Get_5GNR_BWP_Center()/1e6)
            if DMRS:
                odata[2].append("=DMRS=")
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_Mapping())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_1stDMRSSym())

                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_Config())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_AddPosition())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_MSymbLen())

                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenSeed())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_DMRS_RelPwr())
            if PTRS:
                odata[2].append("=PTRS=")
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_PTRS_State())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_PTRS_L())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_PTRS_K())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_PTRS_Pow())
                odata[2].append(self.FSW.Get_5GNR_BWP_Ch_PTRS_RE_Offset())
        except:
            pass
        print('SMW/FSW Values: %d %d'%(len(odata[2]),len(odata[2])))

        return odata

    def Get_5GNR_All_print(self):
        data = self.Get_5GNR_All()
        for i in range(len(data[0])):
            try:
                print(f"{data[0][i]}\t{data[1][i]}\t{data[2][i]}")
            except:
            #     try:
            #         print("%s\t%s\t%s"%(data[0][i],data[1][i],'<notRead>'))
            #     except:
            #         print("%s\t%s\t%s"%(data[0][i],'<notRead>',data[2][i]))
                pass

    def jav_Open(self,SMW_IP,FSW_IP,OFile=''):
        self.SMW = VSG().jav_Open(SMW_IP,OFile)  #Create SMW Object
        self.FSW = VSA().jav_Open(FSW_IP,OFile)  #Create FSW Object
        return self

    def jav_OpenTest(self,SMW_IP,FSW_IP):
        self.SMW = VSG().jav_OpenTest(SMW_IP)  #Create SMW Object
        self.FSW = VSA().jav_OpenTest(FSW_IP)  #Create FSW Object
        return self

    def jav_Close(self):
        self.SMW.jav_Close()
        self.FSW.jav_Close()

    def jav_Clear(self):
        self.SMW.jav_Clear()
        self.FSW.jav_Clear()

    def Set_5GNR_All(self):
        try:
            ### SMW Settings
            self.SMW.cc     = self.NR_CC - 1
            self.SMW.Set_Freq(self.Freq)
            self.SMW.Set_5GNR_BBState('OFF')
            self.SMW.Set_5GNR_Direction(self.NR_Dir)
            self.SMW.Set_5GNR_TransPrecoding(self.NR_TF)
            self.SMW.Set_5GNR_PhaseCompensate(self.NR_PhaseC)
            self.SMW.Set_5GNR_FreqRange(self.NR_Deploy)
            self.SMW.Set_5GNR_ChannelBW(self.NR_ChBW)
            self.SMW.Set_5GNR_BWP_SubSpace(self.NR_SubSp)
            self.SMW.Set_5GNR_BWP_ResBlock(self.NR_RB)
            self.SMW.Set_5GNR_BWP_ResBlockOffset(self.NR_RBO)
            self.SMW.Set_5GNR_BWP_Ch_ResBlock(self.NR_RB)
            self.SMW.Set_5GNR_BWP_Corset_ResBlock(self.NR_RB)
            #self.SMW.Set_5GNR_BWP_Ch_ResBlockOffset(NR_RBO)
            self.SMW.Set_5GNR_BWP_Ch_Modulation(self.NR_Mod)
            self.SMW.Set_5GNR_SSB()
            self.SMW.Set_5GNR_BBState('ON')
            self.SMW.Set_RFState('ON')                            #Turn RF Output on
            self.SMW.Set_RFPwr(self.SWM_Out)                      #Output Power
        except:
            print("NR5G_SetSettings: SMW Error")

        try:
            ### FSW Setting
            # self.cc         = self.NR_CC
            self.FSW.Init_5GNR()
            self.FSW.Set_Freq(self.Freq)
            self.FSW.Set_5GNR_Direction(self.NR_Dir)
            self.FSW.Set_5GNR_TransPrecoding(self.NR_TF)
            self.FSW.Set_5GNR_PhaseCompensate(self.NR_PhaseC)
            self.FSW.Set_5GNR_FreqRange(self.NR_Deploy)
            self.FSW.Set_5GNR_ChannelBW(self.NR_ChBW)
            self.FSW.Set_5GNR_BWP_SubSpace(self.NR_SubSp)
            self.FSW.Set_5GNR_BWP_ResBlock(self.NR_RB)
            self.FSW.Set_5GNR_BWP_ResBlockOffset(self.NR_RBO)
            self.FSW.Set_5GNR_BWP_Ch_ResBlock(self.NR_RB)
            self.FSW.Set_5GNR_BWP_Corset_ResBlock(self.NR_RB)
            #self.FSW.Set_5GNR_BWP_Ch_ResBlockOffset(self.NR_RBO)
            self.FSW.Set_5GNR_BWP_Ch_Modulation(self.NR_Mod)
            self.FSW.Set_SweepCont(1)
            self.FSW.Set_InitImm()
        except:
            print("NR5G_SetSettings: FSW Error")
        return 0


##########################################################
### Instrument Settings
##########################################################
if __name__ == "__main__":
    NR5G = VST().jav_Open('192.168.1.114','192.168.1.109')
    NR5G.Get_5GNR_All_print()
    NR5G.jav_Close()



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 22 22:01:25 2019

@author: RAMIAN

Description:
# Python code to convert from one IQ format to another.
  - Supported formats: iq.tar; wv; iqw

# Functions
  - self.iqData: Complex IQ Data
  - self.read<format>
    - Reads data into self.iqData
    - Sets self.fSamplingRate
    - Sets self.NumberOfSamples
    self.write<format>  Writes self.iqData into <format>
"""

import os
import re
import math
import time
# import array                          # dataConvert
import struct
from datetime import datetime           # writeXml, writeWv
import tarfile                          # readIqTar, writeIqTar
import xml.etree.ElementTree as ET      # readIqTar

class IQ(object):
    """ Util IQ Converting Object """
    def __init__(self):
        self.iqData = [complex(1,1)]    #Complex IQ Data
        self.iiqqList = []
        self.iqiqList = []
        self.NumberOfSamples = 0
        self.fSamplingRate = 0

    def __iqiq2complex__(self, iqiq):
        """iqiq list --> complex list, self.iqData """
        if len(iqiq) % 2 > 0:
            print("Expecting IQIQIQ order, input vector has odd number of samples!")
            return
        self.NumberOfSamples = len(iqiq) // 2
        self.iqData = [complex(iqiq[2*n], iqiq[2*n+1]) for n in range(self.NumberOfSamples)]

    def __iiqq2complex__(self, iiqq):
        """Returns a complex list of I/Q samples from a single list containing IIIQQQ values"""
        if len(iiqq) % 2 > 0:
            print("Expecting IIIQQQ order, input vector has odd number of samples!")
            return
        self.NumberOfSamples = len(iiqq) // 2
        self.iqData = [complex(iiqq[n], iiqq[n+self.NumberOfSamples]) for n in range(self.NumberOfSamples)]

    def __complex2iqiq__(self):
        """Returns a list of I/Q samples from a complex list.
        iqiqlist = __complex2iqiq__(complexList)"""
        f= lambda iq,i: iq.real if i==0 else iq.imag
        self.iqiqList = [f(iq,i) for iq in self.iqData for i in range(2)]

        return self.iqiqList

    # def __complex2iiqq__(self):
    #     """Returns a list of I/Q samples from a complex list.
    #     iiqqList = __complex2iiqq__(complexList)"""
    #     self.iiqqList = [ iq.real for iq in self.iqData]
    #     self.iiqqList.append([ iq.imag for iq in self.iqData])
    #     return self.iiqqList

    def writeIqw(self, FileName):                                               # Verified 2020.0115
        """writes an IQW file (file of binary floats).
        self.iqData can be a list of complex or list of floats.
        Note: IIIQQQ is a deprecated format, don't use it for new files.
        writtenSamples = writeIqw(iqList, "MyFile.wv")"""

        #check if self.iqData is complex
        if isinstance(self.iqData[0], complex):
            self.__complex2iqiq__()
        self.NumberOfSamples = len(self.iqiqList) // 2
        try:
            file = open(FileName, "wb")
            file.write(struct.pack("f"*len(self.iqiqList),*self.iqiqList))
            file.close()
        except:
            print("File (" + FileName +") write error!")
            return 0
        return self.NumberOfSamples

    def readIqw(self, FileName, iqiq = True):                                   # Verified 2020.0115
        """Reads an IQW (iiqq or iqiq) file--> self.__iXXq2complex__ --> self.iqData
         - If iqiq is True, samples are read pairwise (IQIQIQ)
         - else, samples are read, i first then q (IIIQQQ)
        Note: IIIQQQ is a deprecated format, don't use it for new files. """

        print("*.IQW file does not have sampling rate.  Please add to output")
        BytesPerValue = 4
        try:
            file = open(FileName, "rb")
            data = file.read()
            file.close()
        except:
            print("File open error ("+ FileName+")!")

        ReadSamples = len(data) // BytesPerValue
        data = list(struct.unpack("f"*ReadSamples, data))
        if iqiq:
            self.__iqiq2complex__(data)
        else:
            self.__iiqq2complex__(data)

    def writeWv(self, FileName):                # Verified 2020.0115
        """writes a WV file.
        self.iqData can be a list of complex or list of floats (iqiqiq format)."""

        #check if self.iqData is complex
        if isinstance(self.iqData[0], complex):
            self.__complex2iqiq__()

        self.NumberOfSamples = len(self.iqiqList) // 2

        #Find maximum magnitude and scale for max to be FullScale (1.0)
        power = []
        for n in range(self.NumberOfSamples):
            power.append(abs(self.iqiqList[2*n]**2 + self.iqiqList[2*n+1]**2))
        scaling = math.sqrt(max(power))

        self.iqiqList = [iq / scaling for iq in self.iqiqList]                  # normalize to magnitude 1
        rms = math.sqrt(sum(power)/self.NumberOfSamples)/scaling                # calculate rms in dB (below full scale)
        rms = abs(20*math.log10(rms))                                           # Convert to dB
        self.iqiqList = [math.floor(iq * 32767 +.5) for iq in self.iqiqList]    # Convert to int16

        try:
            file = open(FileName, "wb")
            file.write("{TYPE: SMU-WV,0}".encode("ASCII"))
            file.write("{COMMENT: R&S WaveForm, TheAE-RA}".encode("ASCII"))
            file.write(("{DATE: " + str(datetime.today())+ "}").encode("ASCII"))
            file.write(("{CLOCK:" +str(self.fSamplingRate) + "}").encode("ASCII"))
            file.write(("{LEVEL OFFS:" +  "{:2.4f}".format(rms) + ",0}").encode("ASCII"))
            file.write(("{SAMPLES:" + str(self.NumberOfSamples) + "}").encode("ASCII"))
        #    if(m1start > 0 && m1stop > 0)
        #        %Control Length only needed for markers
        #        fprintf(file_id,'%s',['{CONTROL LENGTH:' num2str(data_length) '}']);
        #        fprintf(file_id,'%s',['{CLOCK MARKER:' num2str(fSamplingRate) '}']);
        #        fprintf(file_id,'%s',['{MARKER LIST 1: ' num2str(m1start) ':1;' num2str(m1stop) ':0}']);
        #    end
            file.write(("{WAVEFORM-" +  str(4*self.NumberOfSamples+1) + ": #").encode("ASCII"))
            file.write(struct.pack("h"*len(self.iqiqList),*self.iqiqList))
            file.write("}".encode("ASCII"))
            file.close()
        except:
            print("File (" + FileName +") write error!")
            return 0
        return self.NumberOfSamples

    def readWv(self,FileName):                                                          # Verified 2020.0115
        """Reads a WV file --> self.__iqiq2complex__ --> self.iqData"""
        try:
            file = open(FileName, "rb")
            data = file.read()
            file.close()
        except:
            print("File open error ("+ FileName+")!")
            return

        binaryStart = 0
        tags = ""
        Counter = 0
        ConverterSize = 20
        while (binaryStart == 0) & (Counter < len(data)):
            tags += data[Counter:Counter+ConverterSize].decode("ASCII","ignore")
            Counter += ConverterSize
            res = re.search("WAVEFORM.{0,20}:.{0,3}#",tags)
            if res is not None:
                binaryStart = res.span()[1]

        if (Counter > len(data)) & (binaryStart == 0):
            print("Required tags not found, potentially incompatible file format!")
            return

        res = re.search("SAMPLES[ ]*:[ ]*(?P<NumberOfSamples>[0-9]*)",tags)
        self.NumberOfSamples = int(res.group("NumberOfSamples"))
        res = re.search("CLOCK[ ]*:[ ]*(?P<SamplingRate>[0-9]*)",tags)
        self.fSamplingRate = float(res.group("SamplingRate"))
        data = list(struct.unpack("h"*self.NumberOfSamples*2, data[binaryStart:-1]))    #MMM data: IQ arry
        data = list(map(lambda x: x/32767.0, data))                                   #MMM consumes a lot of time
        self.__iqiq2complex__(data)

    def writeXml(self, filenameiqw, filenamexml):                                       # Verified  2020.0115
        """Function to write the xml part of the iq.tar"""
        xmlfile = open(filenamexml, "w")
        xmlfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
        xmlfile.write("<?xml-stylesheet type=\"text/xsl\" href=\"open_IqTar_xml_file_in_web_browser.xslt\"?>\n")
        xmlfile.write("<RS_IQ_TAR_FileFormat fileFormatVersion=\"2\" xsi:noNamespaceSchemaLocation=\"http://www.rohde-schwarz.com/file/RsIqTar.xsd\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n")
        xmlfile.write("<Name>Python iq.tar writer (self.iqData.py)</Name>\n")           #Optional
        xmlfile.write("<Comment>RS WaveForm, TheAE-RA</Comment>\n")                     #Optional
        xmlfile.write("<DateTime>"+ datetime.now(None).isoformat() +"</DateTime>\n")
        xmlfile.write("<Samples>" + str(self.NumberOfSamples) + "</Samples>\n")
        xmlfile.write("<Clock unit=\"Hz\">" + str(self.fSamplingRate) + "</Clock>\n")
        xmlfile.write("<Format>complex</Format>\n")
        xmlfile.write("<DataType>float32</DataType>\n")
        xmlfile.write("<ScalingFactor unit=\"V\">1</ScalingFactor>\n")                  #Optional
        xmlfile.write("<NumberOfChannels>1</NumberOfChannels>\n")                       #Optional
        xmlfile.write("<DataFilename>" + filenameiqw+ "</DataFilename>\n")
        # xmlfile.write("<UserData></UserData>\n")                                      #Optional
        xmlfile.write("</RS_IQ_TAR_FileFormat>\n")
        xmlfile.close()
        return 1

    def writeIqTar(self, FileName):             # Verified 2020.0115  #pylint: disable=R1710
        """writes an iq.tar file. Complex self.iqData values are interpreted as Volts.
        self.iqData can be a list of complex or list of floats (iqiqiq format)."""
        path,filename = os.path.split(FileName)                                     #pylint: disable=W0612

        #Create binary file
        binaryfile = re.sub("iq.tar", "complex.1ch.float32", filename, flags=re.IGNORECASE)
        self.writeIqw(os.path.join(path, binaryfile))
        if self.NumberOfSamples == 0:
            return 0
        # xsltfilename = "open_IqTar_xml_file_in_web_browser.xslt"                  #xslt is optional
        xmlfilename = binaryfile + '.xml'
        self.writeXml(binaryfile, os.path.join(path, xmlfilename))

        try:
            tar = tarfile.open(FileName, "w")
            tar.add(os.path.join(path, binaryfile), arcname=binaryfile)
            #tar.add(os.path.join(path, xsltfilename), arcname=xsltfilename)         #xslt is optional
            tar.add(os.path.join(path, xmlfilename), arcname=xmlfilename)
            tar.close()
            os.remove(os.path.join(path, binaryfile))
            os.remove(os.path.join(path, xmlfilename))
        except:
            print("IqTar (" + FileName +") write error!")
            return 0

    def readIqTar(self,FileName):               # Verified 2020.0115
        """Reads an iq.tar file --> self.iqData"""
        path,filename = os.path.split(FileName)                         #pylint: disable=W0612
        self.fSamplingRate = 0

        try:
            tar = tarfile.open(FileName, "r:")
            a=tar.getnames()
            xmlfile = [filename for filename in a if ".xml" in filename.lower()]
            xmlfile = xmlfile[0]
            tar.extract(xmlfile)
            tree = ET.parse(xmlfile)
            root = tree.getroot()
            binaryfilename = root.find("DataFilename").text
            self.fSamplingRate = float(root.find("Clock").text)
            helper = root.find("ScalingFactor")
            scaling = 1
            if helper:
                if helper.get("unit")!="V":
                    print("Only (V)olts scaling factor supported - assuming 1V!")
                else:
                    scaling = float(root.find("ScalingFactor").text)
            os.remove(xmlfile)
            del root
            tar.extract(binaryfilename)
            tar.close()
            self.readIqw(binaryfilename)
            os.remove(binaryfilename)
        except:
            print("IqTar (" + FileName +") read error!")

        self.iqData = [sample * scaling for sample in self.iqData]                      #Apply scaling factor

    def main(self):     #pragma: no cover
        #for testing only
        filename = "C:\\Users\\lim_m\\ownCloud\\ATE\\AA_Code\\DSP_python\\DSP_python\\SampleWv\\CW_10tones_32MHz_100usec.iq.tar"

        ### Read data
        if '.wv' in filename:
            self.readWv(filename)
        elif '.iq.tar' in filename:
            self.readIqTar(filename)
        elif '.iqw' in filename:
            self.readIqw(filename)
        else:
            print('Filetype not supported')

        start = time.time()
        self.writeIqTar(filename + '.iq.tar')
        self.writeWv(filename + '.wv')
        self.writeIqw(filename + '.iqw')
        duration = time.time() - start
        speed = self.NumberOfSamples/1e6/duration
        print("Total: %d samples in %2.2f ms. writeSpeed: %f MSamples/s"%(self.NumberOfSamples,duration*1e3,speed))

def dataConvert():                          #pragma: no cover
    ''' binary to float32 conversion '''
    # numpy.frombuffer dtype=float32()
    sampledata = b'\x0E\x78\x94\xB7'
    print('Data %s %d'%(sampledata,len(sampledata)))
    print("Float=",struct.unpack("<f",sampledata))  #Little Endian
    print("Float=",struct.unpack("<f",sampledata))  #Little Endian
    # print("Float=",array.frombytes(sampledata))   #Little Endian

if __name__ == "__main__":
    # IQ().main()
    dataConvert()



###############################################################################
### Purpose: rssd.self.NRP.common.py driver test
###              _   ___        __  _____         _   
###             | | | \ \      / / |_   _|__  ___| |_ 
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_ 
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.40'              #Get local machine name
#host = 'NRP8-101507.local'                #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.NRP.Common import PMr

class TestGeneral(unittest.TestCase):
    def setUp(self):                      #run before each test
        self.NRP8 = PMr().jav_OpenTest(host)

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.NRP8.jav_Error()[0],'0')
        self.NRP8.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_NRP_Gets(self):
        self.NRP8.Get_BufferedMeas('ON')
        self.NRP8.Get_BufferedMeas('OFF')
        self.NRP8.Get_EventStatus()
        self.NRP8.Get_Offset()

    def test_NRP_List(self):
        self.NRP8.Get_AvailableNRP()

    def test_NRP_Freq(self):
        SetVal = 1e9
        self.NRP8.Set_Freq(SetVal)
        GetVal = self.NRP8.Get_Freq()
        if self.NRP8.connected: self.assertEqual(SetVal,int(GetVal))

    def test_NRP_Power(self):
        self.NRP8.Set_Freq(6e9)                              # Set Frequency
        self.NRP8.Set_Function('AVG')
        self.NRP8.Set_InitCont('OFF')
        self.NRP8.Set_AverageMode(1)                         # Auto Averaging ON
        self.NRP8.Set_Average(10)                            # Avg Count = 4
        self.NRP8.Set_PowerOffset(5)
        self.NRP8.Set_PowerOffsetState(1)
        self.NRP8.Set_InitImm()
        self.NRP8.Set_AverageMode(0)                         # Auto Averaging OFF
        self.NRP8.Set_PowerOffsetState(0)
        GetVal = self.NRP8.Get_Average()
        GetVal = self.NRP8.Get_Power()

    def test_NRPM_Power(self):
        self.NRP8.Set_Freq(6e9)                              # Set Frequency
        self.NRP8.Set_AverageMode(1)                         # Auto Averaging OFF
        self.NRP8.Set_Average(10)                            # Avg Count = 4
        self.NRP8.Set_PowerOffset(5)
        self.NRP8.Set_PowerOffsetState(1)
        self.NRP8.Set_InitImm()
        self.NRP8.Set_NRPM_LED(1)
        self.NRP8.Set_NRPM_LED(0)
        GetVal = self.NRP8.Get_NRPM_PowerAll()

    def test_NRP_Trigger(self):
        self.NRP8.Set_TriggerAuto(0)
        self.NRP8.Set_TriggerCount(10)
        self.NRP8.Set_TriggerSource('BUS')
        self.NRP8.Set_TriggerAuto(1)

    def test_NRP_Send(self):
        self.NRP8.Set_BufferSize(12)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_NRP_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=4).run(suite)



##########################################################
### Rohde & Schwarz Automation for demonstration use.
### Purpose: VSG List Mode
### Author:  mclim
### Date:    2019.06.13
##########################################################
### User Entry
##########################################################
host = '192.168.1.114'               #Get local machine name

##########################################################
### Code Start
##########################################################
from rssd.SMW_Common import VSG

SMW = VSG().jav_Open(host)
#SMW.jav_logSCPI()                  # Log SCPI commands
SMW.Set_Freq(10e9)                  # Set 10GHz
SMW.Set_RFPwr(-30)                  # Output -30dBm
SMW.Set_ListMode_File('list1.lsw')  # Select List File
SMW.Set_ListMode_TrigSource('SING')  # Single run
SMW.Set_ListMode_RMode('LIVE')
SMW.Set_ListMode_Dwell(1)           # Dwell time
SMW.Set_RFState(1)                  # RF Output on.
SMW.Set_ListMode('LIST')            # Turn on listmode
SMW.Set_ListMode_TrigExecute()      # Execute single trigger
SMW.Set_ListMode_TrigWait()         # Wait for single trigger
SMW.Set_RFState('OFF')              # RF Output off

SMW.jav_ClrErr()                    # Clear Errors



from __future__ import division         #int div to float
########################################################################
# Title: Rohde & Schwarz Simple ATE example GUI
########################################################################
# User Input Settings
########################################################################
btnWid = 11
Col0Wid = 10                            #Text Labels
Col1Wid = 14                            #Text Input
ColxWid = 4*(btnWid+4) -4
BotWindWid = Col0Wid+Col1Wid+ColxWid
maxCol = 6
btnRow = 20
ClrTxtBg     = "black"                  #gray30
ClrTxtFg     = "green2"
BtnTxtFg     = "white"
ColorCurs    = "white"
ClrAppBG     = "grey30"

########################################################################
### Code Import
########################################################################
try:        #Python2 naming
    import Tkinter              as      Tk
    import ttk
    import tkFileDialog
except:    #Python3 naming
    import tkinter              as      Tk
    from    tkinter             import  ttk
    import tkinter.filedialog   as      tkFileDialog
END = Tk.END

#Code specific libraries
import  copy
from    datetime  import datetime
from    yaVISA     import jaVisa

########################################################################
### Functions
########################################################################
class GUIData(object):
    def __init__(self):
        self.Title  = "Rohde&Schwarz VISA Utility"
        self.EntTxt1= "Instrument IP"
        self.Entry1 = "127.0.0.1"
        self.EntTxt2= "SCPI String"
        self.Entry2 = "*IDN?"
        self.EntTxt3= "Entry 3"
        self.Entry3 = ""
        self.EntTxt4= "Entry 4"
        self.Entry4 = ""
        self.List1  = ['*IDN?','*OPT?']
        self.BtnTxt1= 'VISA Scan'
        self.BtnTxt2= 'SCPI List'
        self.BtnTxt3= '*IDN?'
        self.BtnTxt4= 'Query'
        self.BtnTxt5= 'Write'

def ArrayInput(stringIn):
    OutputList = []
    InputList = stringIn.split(",")
    for i in InputList:
        i = i.strip()
        OutputList.append(i)
    return OutputList

def btn_Clear():
    # posi = lstBotWind.curselection()
    lstBotWind.delete(0.0,END)

def btn1():
    """Scan VISA"""
    windowLowerWrite("[Resource List]")
    K2 = jaVisa()
    InstrList = K2.jav_reslist()
    for i in InstrList:
        windowLowerWrite("    " + i)
    K2.jav_Close()

def btn2():
    """Send SCPIList in TopWindow"""
    windowLowerWrite("[SCPI LIST]")
    K2 = jaVisa()
    K2.jav_Open(Entry1.get())
    SCPIList = lstTopWind.get("1.0",END).split('\n')
    OutList = K2.jav_scpilist(SCPIList)
    try:
        windowLowerWrite('  Err:' + ''.join(K2.jav_Close()))
    except:
        pass

    for Ostr in OutList:
        windowLowerWrite("  " + Ostr)

def btn3():
    """*IDN Query"""
    windowLowerWrite("--> *IDN?")
    K2 = jaVisa()
    K2.jav_Open(Entry1.get())
    readStr = K2.query("*IDN?")
    windowLowerWrite("<-- " + readStr)
    K2.jav_Close()

def btn4():
    """Query SCPI String"""
    windowLowerWrite("--> " + Entry2.get())
    K2 = jaVisa()
    K2.jav_Open(Entry1.get())
    readStr = K2.query(Entry2.get())
    windowLowerWrite("<-- " + readStr)
    K2.jav_Close()

def btn5():
    """Write SCPI """
    windowLowerWrite("--> " + Entry2.get())
    K2 = jaVisa()
    K2.jav_Open(Entry1.get())
    K2.write(Entry2.get())
    K2.jav_Close()

def dataLoad():
    OutObj = GUIData()
    try:
        try:          #Python3
            f = open(__file__ + ".csv","rt")
        except:      #Python2
            f = open(__file__ + ".csv","rb")
        data = f.read().split(',')
        OutObj.Entry1 = data[0]
        OutObj.Entry2 = data[1]
        OutObj.Entry3 = data[2]
        OutObj.Entry4 = data[3]
        windowLowerWrite("DataLoad: File")
    except:
        windowLowerWrite("DataLoad: Default")
    return OutObj

def dataSave():
    try:    #Python3
        f = open(__file__ + ".csv",'wt', encoding='utf-8')
    except:
        f = open(__file__ + ".csv",'wb')
    f.write('%s,'%(Entry1.get()))
    f.write('%s,'%(Entry2.get()))
    f.write('%s,'%(Entry3.get()))
    f.write('%s,'%(Entry4.get()))
    f.close()
    windowLowerWrite("DataSave: File Saved")

def windowLowerClear():
    # posi = lstBotWind.curselection()
    lstBotWind.delete(0.0,END)

def windowLowerWrite(inStr):
    sDate = datetime.now().strftime("%y%m%d-%H%M%S.%f")             #Date String
    try:
        lstBotWind.insert(Tk.INSERT,"%s %s\n"%(sDate[:-3],inStr))   #Text moves down
        # lstBotWind.insert(END,"%s %s\n"%(sDate[:-3],inStr))         #Text moves up
        # lstBotWind.see(END)                                         #Text moves up
        GUI.update()
    except:
        pass

def windowUpperClear():
    #posi = lstTopWind.curselection()
    lstTopWind.delete(0.0,END)

def windowUpperWrite(inStr):
    lstTopWind.insert(END,'\n'+inStr)
    lstTopWind.see(END)
    GUI.update()

def menu_Exit():
    # global GUI
    dataSave()
    GUI.quit()
    GUI.destroy()
    print("Program End")

def menu_Open():
    asdf = tkFileDialog.askopenfilename()
    print(asdf)

def menu_Save():
    dataSave()

def mnu_TopWindLoad_Files():
    lstTopWind.delete(0,END)
    filez = tkFileDialog.askopenfilenames()
    for i in list(filez):
        lstTopWind.insert(END,i)
    lstTopWind.see(END)

def mnu_TopWindLoad_Read():
    lstTopWind.delete(0,END)
    filez = tkFileDialog.askopenfilename()
    windowLowerWrite(filez)
    for i in list(filez):
        lstTopWind.insert(END,i)
    lstTopWind.see(END)

def mnu_TopWindClear():
    # posi = lstTopWind.curselection()
    lstTopWind.delete(0.0,END)

def mnu_SaveCond():
    dataSave()

########################################################################
### Main Code
########################################################################
RSVar = copy.copy(dataLoad())
GUI = Tk.Tk()                                                           #Create GUI object
GUI.title(RSVar.Title)                                                  #GUI Title
#GUI.iconbitmap('RSIcon.ico')
GUI.resizable(0,0)
GUI.config(bg=ClrAppBG)

########################################################################
### Define GUI Widgets
Label1 = Tk.Label(GUI,width=Col0Wid, bg=ClrAppBG, text=RSVar.EntTxt1)   #Create Label
Entry1 = Tk.Entry(GUI,width=Col1Wid, bg=ClrTxtBg, fg=ClrTxtFg, insertbackground=ColorCurs)
Entry1.insert(END,RSVar.Entry1)                                         #Default Value
Label2 = Tk.Label(GUI,width=Col0Wid, bg=ClrAppBG, text=RSVar.EntTxt2)   #Create Label
Entry2 = Tk.Entry(GUI,width=Col1Wid, bg=ClrTxtBg, fg=ClrTxtFg, insertbackground=ColorCurs)
Entry2.insert(END,RSVar.Entry2)                                         #Default Value
Label3 = Tk.Label(GUI,width=Col0Wid, bg=ClrAppBG, text=RSVar.EntTxt3)   #Create Label
Entry3 = Tk.Entry(GUI,width=Col1Wid, bg=ClrTxtBg, fg=ClrTxtFg, insertbackground=ColorCurs)
Entry3.insert(END,RSVar.Entry3)                                         #Default Value
Label4 = Tk.Label(GUI,width=Col0Wid, bg=ClrAppBG, text=RSVar.EntTxt4)   #Create Label
Entry4 = Tk.Entry(GUI,width=Col1Wid, bg=ClrTxtBg, fg=ClrTxtFg, insertbackground=ColorCurs)
Entry4.insert(END,RSVar.Entry4)                                         #Default Value
btnObj1 = Tk.Button(GUI,width=btnWid,bg=ClrAppBG,fg=BtnTxtFg,text=RSVar.BtnTxt1,command = btn1)
btnObj2 = Tk.Button(GUI,width=btnWid,bg=ClrAppBG,fg=BtnTxtFg,text=RSVar.BtnTxt2,command = btn2)
btnObj3 = Tk.Button(GUI,width=btnWid,bg=ClrAppBG,fg=BtnTxtFg,text=RSVar.BtnTxt3,command = btn3)
btnObj4 = Tk.Button(GUI,width=btnWid,bg=ClrAppBG,fg=BtnTxtFg,text=RSVar.BtnTxt4,command = btn4)
btnObj5 = Tk.Button(GUI,width=btnWid,bg=ClrAppBG,fg=BtnTxtFg,text=RSVar.BtnTxt5,command = btn5)
btnQuit = Tk.Button(GUI,width=btnWid,bg='red'   ,fg=BtnTxtFg,text="Quit",       command = menu_Exit)

########################################################################
### List Boxes
########################################################################
lstBotWind = Tk.Text(GUI, width=BotWindWid, bg=ClrTxtBg, fg=ClrTxtFg,\
                            wrap=Tk.CHAR, height=15)
srlBotWind = ttk.Scrollbar(GUI, orient=Tk.VERTICAL, command=lstBotWind.yview)   #Create scrollbar
lstBotWind.config(wrap='none')
lstBotWind.config(yscrollcommand=srlBotWind.set)                                #Link scroll to lstBotWind
lstBotWind.insert(Tk.INSERT,"Output Window\n")
lstBotWind.tag_add("here", "1.0", "1.40")
lstBotWind.tag_config("here", background="yellow", foreground="blue")

lstTopWind = Tk.Text(GUI,bg=ClrTxtBg, fg=ClrTxtFg, width=ColxWid,height=10)
srlTopWind = ttk.Scrollbar(GUI, orient=Tk.VERTICAL,command=lstTopWind.yview)   #Create scrollbar
for item in RSVar.List1:
    lstTopWind.insert(END, item + '\n')
lstTopWind.config(yscrollcommand=srlTopWind.set)                               #Link scroll to lstTopWind

########################################################################
### Draw Widgets w/ Grid
########################################################################
Label1.grid(row=0,column=0,sticky=Tk.E)
Label2.grid(row=1,column=0,sticky=Tk.E)
#Label3.grid(row=2,column=0,sticky=Tk.E)
#Label4.grid(row=3,column=0,sticky=Tk.E)
Entry1.grid(row=0,column=1)
Entry2.grid(row=1,column=1)
#Entry3.grid(row=2,column=1)
#Entry4.grid(row=3,column=1)
btnObj1.grid(row=btnRow,column=0)
btnObj2.grid(row=btnRow,column=1)
btnObj3.grid(row=btnRow,column=2)
btnObj4.grid(row=btnRow,column=3)
btnObj5.grid(row=btnRow,column=4)
btnQuit.grid(row=btnRow,column=5)

lstTopWind.grid(row=0,       column=2,      columnspan=4,rowspan=4, sticky=(Tk.E))
srlTopWind.grid(row=0,       column=maxCol, rowspan=4,              sticky=(Tk.W,Tk.N,Tk.S))
lstBotWind.grid(row=btnRow-1,column=0,      columnspan=(maxCol),    sticky=Tk.E)
srlBotWind.grid(row=btnRow-1,column=maxCol,                         sticky=(Tk.W,Tk.N,Tk.S))

########################################################################
# Define menu
########################################################################
menu = Tk.Menu(GUI)                                                 #create GUI dropdown
GUI.config(menu=menu)                                               #define GUI's menu

fileMenu = Tk.Menu(menu)                                            #create dropdown menu
fileMenu.add_command(label="Open",command=menu_Open)
fileMenu.add_command(label="Save",command=menu_Save)
fileMenu.add_separator()
fileMenu.add_command(label="SCPI Load", command=mnu_TopWindLoad_Read)
fileMenu.add_command(label="SCPI Clear",command=mnu_TopWindClear)
fileMenu.add_separator()
fileMenu.add_command(label="Exit",command=menu_Exit)
menu.add_cascade(label="File",menu=fileMenu)                        #add dropdown menu

editMenu = Tk.Menu(menu)                                            #create dropdown menu
editMenu.add_command(label="Edit",command=menu_Open)
menu.add_cascade(label="Edit",menu=editMenu)                        #add dropdown menu

########################################################################
# Start Program
########################################################################
GUI.mainloop()         #Display window



"""Timing SCPI Commands Example"""
import timeit
from rssd.instrument        import instr

🤖 = instr().open('192.168.58.109')
🤖.timeout(1)

def timeQuery(SCPI):
    tick = timeit.default_timer()
    rdStr = 🤖.query(SCPI)
    TotTime = timeit.default_timer() - tick
    outStr = f'{TotTime*1000:.3f}mSec,{rdStr}'
    return outStr

print(timeQuery('*IDN?'))
# print(timeQuery(':SENS:FREQ:CENT 24e9;*OPC?'))
# print(timeQuery(':INIT:IMM;*OPC?'))

🤖.close()



"""
Vector Signal Generator LTE Functions
### Options: K55  LTE Rel  8: LTE 20MHz 64QAM 
###          K84  LTE Rel  9: MBMS & MBSFN generation
###          K85  LTE Rel 10: CA; Enhanced SC-FDMA; UL MIMO
###          K112 LTE Rel 11: TDD Config9&7
###          K113 LTE Rel 12: DL 256QAM; TDD/FDD Mixed CA
###          K119 LTE Rel 13/14/15: UL 256QAM; LAA
#####################################################################
"""
from rssd.VSG.Common import VSG     #pylint: disable=E0611,E0401

class VSG(VSG):                     #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Generator LTE Object """
    def __init__(self):
        super(VSG,self).__init__()    #Python2/3
        self.Model  = "SMW"
        self.ldir   = "UL"
        self.User   = 0
        self.cc     = 1

    #####################################################################
    ### VSG Query
    #####################################################################
    def Get_LTE_CC(self):
        rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:CA:CELL0:STAT?')
        return rdStr

    def Get_LTE_ChBW(self):
        rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:BW?')
        return rdStr

    def Get_LTE_Direction(self):
        rdStr = self.query(f':SOUR:BB:EUTR:LINK?')
        if rdStr == 'DOWN':
            self.ldir = "DL"
        elif rdStr == 'UP':
            self.ldir = "UL"
        else:
            print('Get_LTE_Direction Error')
        return self.ldir

    def Get_LTE_Duplex(self):
        rdStr = self.query(f':SOUR:BB:EUTR:DUPL?')
        return rdStr

    def Get_LTE_Modulation(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:CW1:PUSC:MOD?')     # UL Only
        else:
            rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:DUMD:MOD?')
        return rdStr

    def Get_LTE_ResBlock(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:PUSC:SET1:RBC?')    # UL Only
        else:
            rdStr = self.query(f':SOUR:BB:EUTR:DL:SUBF0:ALL0:CW1:RBC?')                         # DL Only
        return rdStr

    def Get_LTE_ResBlockOffset(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:PUSC:SET1:VRB?')    # UL Only
        else:
            rdStr = self.query(f':SOUR:BB:EUTR:DL:SUBF0:ALL0:CW1:RBOF?')                        # DL Only
        return rdStr

    #####################################################################
    ### VSG Setting
    #####################################################################
    def Set_LTE_BBState(self,iEnable):
        if (iEnable == 1) or (iEnable == 'ON'):
            self.jav_OPC_Wait(':SOUR1:BB:EUTR:STAT 1')
    #         self.query('*OPC?')        # Wait for calculation
        else:
            self.write(':SOUR1:BB:EUTR:STAT 0')

    def Set_LTE_CC(self,iCC):
        # self.write(''%())
        if iCC > 0:
            self.write(f':SOUR:BB:EUTR:{self.ldir}:CA:STAT 1')

    def Set_LTE_ChBW(self,iChBW):
        self.write(f':SOUR:BB:EUTR:{self.ldir}:BW BW{iChBW:02d}_00')
        self.write(f':SOUR:BB:EUTR:{self.ldir}:CA:CELL{self.cc - 1}:BW BW{iChBW:02d}_00')

    def Set_LTE_Direction(self, sDir):
        """UL | DL | UP | DOWN """
        if (sDir == 'UL') or (sDir == 'UP'):
            self.write(f':SOUR:BB:EUTR:LINK UP')
            self.ldir = 'UL'
        elif (sDir == 'DL') or (sDir == 'DOWN'):
            self.write(f':SOUR:BB:EUTR:LINK DOWN')
            self.ldir = 'DL'
        else:
            print('Set_LTE_Direction Error.  Must be UL or DL')

    def Set_LTE_Duplex(self,sDuplex):
        #FDD TDD
        self.write(f':SOUR:BB:EUTR:DUPL {sDuplex}')

    def Set_LTE_Modulation(self, sMod):
        if self.ldir == 'UL':
            self.write(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:CW1:PUSC:MOD {sMod}')
        else:
            self.write(f':SOUR:BB:EUTR:{self.ldir}:DUMD:MOD {sMod}')

    def Set_LTE_ResBlock(self, iRB):
        if self.ldir == 'UL':
            self.write(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:PUSC:SET1:RBC {iRB}')
        else:
            pass

    def Set_LTE_ResBlockOffset(self,iRBO):
        if self.ldir == 'UL':
            self.write(f':SOUR:BB:EUTR:{self.ldir}:CELL0:SUBF0:ALL0:PUSC:SET1:VRB {iRBO}')
        else:
            pass



###############################################################################
### Purpose: rssd.VSA.LTE_K100 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.VSA_K70      import VSA

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_VSA()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_VSA_Auto(self):
        self.FSW.Set_VSA_Capture_Length(0)
        self.FSW.Set_VSA_Capture_Time(0)

    def test_FSW_VSA_EQ(self):
        self.FSW.Set_VSA_EqualizerState(0)
        getVal = self.FSW.Get_VSA_EqualizerState()
        if self.FSW.connected: self.assertEqual(getVal, '0')
        self.FSW.Set_VSA_EqualizerState('OFF')
        getVal = self.FSW.Get_VSA_EqualizerState()
        if self.FSW.connected: self.assertEqual(getVal, '0')
        self.FSW.Set_VSA_EqualizerState(1)
        getVal = self.FSW.Get_VSA_EqualizerState()
        if self.FSW.connected: self.assertEqual(getVal, '1')
        self.FSW.Set_VSA_EqualizerState('ON')
        getVal = self.FSW.Get_VSA_EqualizerState()
        if self.FSW.connected: self.assertEqual(getVal, '1')

    def test_FSW_VSA_Get(self):
        self.FSW.Get_VSA_IQImbalance()
        self.FSW.Get_VSA_GainImbalance()
        # self.FSW.Get_VSA_IQSkew()
        self.FSW.Get_VSA_MER()
        self.FSW.Get_VSA_Rho()
        self.FSW.Get_VSA_ResultSumamry()
        self.FSW.Get_VSA_symbol_rate()
        # self.FSW.Get_VSA_SymbolRateError()

    def test_FSW_VSA_Meas(self):        #error
        self.FSW.Set_VSA_Symbol_Rate(1e6)
        self.FSW.Set_VSA_Filter_Type('RRC')
        self.FSW.Set_VSA_Filter_Alpha(0.2)
        self.FSW.Set_VSA_Capture_Time(1e-3)
        self.FSW.Set_VSA_Capture_Length(500)
        self.FSW.Set_VSA_EqualizerState(1)
        self.FSW.Set_VSA_Result_Length(200)
        self.FSW.Set_VSA_Result_Length('MAX')
        self.FSW.Get_VSA_Meas_Params()

    def test_FSW_VSA_Mod(self):
        self.FSW.Set_VSA_Mod('QPSK')
        self.FSW.Set_VSA_Mod('OQPSK')
        self.FSW.Set_VSA_Mod('8PSK')
        self.FSW.Set_VSA_Mod('16APSK')
        self.FSW.Set_VSA_Mod('32APSK')
        self.FSW.Set_VSA_Mod('64APSK')
        self.FSW.Set_VSA_Mod('256APSK')

    def test_FSW_VSA_Mod_QAM(self):
        self.FSW.Set_VSA_Mod_Type('QAM')
        self.FSW.Set_VSA_Mod_QAM(16)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_VSA_VSA
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



##########################################################
### Rohde & Schwarz Automation for demonstration use.
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date   : 2018.05.24
##########################################################
### User Entry
##########################################################
VSA_IP  = '192.168.1.108'
VSG_IP  = '192.168.1.114'
MeasTim = 10e-3
Freq    = 28e9
RBW     = 200e3
ChBW    = 95e6
ChSpace = 100e6
Avg     = 0
SweMode = 'AUTO'            #AUTO | SPEed | DYN
SweType = 'AUTO'           #AUTO | SWE | FFT

### Loops
Repeat  = 10
PwrSweep = 59

##########################################################
### Code Overhead
##########################################################
from datetime               import datetime
from rssd.VSA.Common        import VSA              #pylint: disable=E0611,E0401
from rssd.yaVISA_socket     import jaVisa           #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO           #pylint: disable=E0611,E0401

OFile = FileIO().makeFile(__file__)
VSA = VSA().jav_Open(VSA_IP,OFile)                  #Create VSA Object
VSG = jaVisa().jav_Open(VSG_IP,OFile)               #Create Object

meth = {0:'Autolvl()',
        1:'TxChOnly()',
        2:'AvgOff()',
        3:'AutoRBW()',
        4:'SwpTime()',
        5:'SetReflvl()',
        6:'K18()'
        }
meth = {1:'SetReflvl()',
        0:'K18()',
        }
asdf=len(meth)

def Autolvl():
    VSA.query(':SENS:ADJ:LEV;*OPC?')                        # Auto-Tune

def TxChOnly():
    VSA.Set_ACLR_NumAdj(0)
    VSA.query(':SENS:ADJ:LEV;*OPC?')                        # Auto-Tune
    VSA.Set_ACLR_NumAdj(2)

def AvgOff():
    VSA.Set_Trace_AvgCount(1)
    VSA.query(':SENS:ADJ:LEV;*OPC?')                        # Auto-Tune
    VSA.Set_Trace_AvgCount(Avg)

def AutoRBW():
    VSA.Set_ResBW(0)
    VSA.query(':SENS:ADJ:LEV;*OPC?')                        # Auto-Tune
    VSA.Set_ResBW(RBW)

def SwpTime():
    VSA.Set_SweepTime(2e-3)
    VSA.query(':SENS:ADJ:LEV;*OPC?')                        # Auto-Tune
    VSA.Set_SweepTime(MeasTim)

def NoAutoLvl():
    pass

def SetReflvl():
    VSA.Set_AttnAuto()
    VSA.write(':INIT:CONT OFF')                             # Single Sweep
    VSA.Set_InitImm()
    ChPwr = VSA.queryFloatArry(':CALC:MARK:FUNC:POW:RES? MCAC')[0]
    VSA.Set_RefLevel(ChPwr + 3)
    VSA.Set_InitImm()
    ChPwr = VSA.queryFloatArry(':CALC:MARK:FUNC:POW:RES? MCAC')[0]
    VSA.Set_RefLevel(ChPwr + 3)
    if ChPwr < -24:     #FSVA:-17  FSW:-22
        VSA.Set_Preamp('ON')
    else:
        VSA.Set_Preamp('OFF')

def K18():
    VSA.Set_Channel('AMPL')
    VSA.Set_Autolevel()                             # Auto-Tune

##########################################################
### Code Start
##########################################################
# VSA.jav_Reset()
VSA.Set_Freq(Freq)
VSA.Init_ACLR()                                     #VSA ACLR Channel
VSA.Set_ACLR_CHBW(ChBW)
VSA.Set_ACLR_AdjBW(ChBW)
VSA.Set_ACLR_AdjSpace(ChSpace)
VSA.Set_ACLR_NumAdj(2)

VSA.Set_ResBW(RBW)
#VSA.Set_DisplayUpdate("OFF")
VSA.Set_SweepTime(MeasTim)
VSA.Set_Trace_Avg('LIN')
VSA.Set_Trace_AvgCount(Avg)
VSA.Set_Trace_Detector('RMS')
VSA.Set_SweepOpt(SweMode)
VSA.Set_SweepType(SweType)
VSA.Set_In_YIG('OFF')
# VSA.Set_Trig1_Source('Ext')

##########################################################
### Measure Time
##########################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
OFile.write('Instr,Iter,RBW,SwpTime,SMWPwr,ALType,AL Time,TotalTime,Attn,PreAmp,RefLvl,SwpTime,SwpPts,SwpType,SwpOpt,TxPwr,Adj-,Adj+,Alt-,Alt+,Freq,ChBw')
for i in range(Repeat):
    for autoMeth in meth:
        for pwr in range(PwrSweep):
            ### <\thing we are timing>
            VSG.query(f':POW:AMPL {-50 + pwr}dbm;*OPC?')            # VSG Power
            tick = datetime.now()

            ### <AUTOLEVEL> ###
            eval(autoMeth)                                          # Dynamically call #pylint: disable=W0123
            ### <AUTOLEVEL> ###

            tockA = datetime.now()
            VSA.write(':INIT:CONT OFF')                             # Single Sweep
            VSA.query(':INIT:IMM;*OPC?')                            # Take Sweep
            ACLR = VSA.query(':CALC:MARK:FUNC:POW:RES? MCAC')
            ### <\thing we are timing>
            tockB = datetime.now()
            SwpParam = VSA.Get_SweepParams()
            AmpSet  = VSA.Get_AmpSettings()
            ALTime = tockA - tick
            TotTime = tockB - tick
            OutStr = f'{VSA.Model},{i},{RBW},{MeasTim},{-50+pwr},{meth[autoMeth]},{ALTime.seconds:3d}.{ALTime.microseconds:06d},{TotTime.seconds:3d}.{TotTime.microseconds:06d},{AmpSet},{SwpParam},{ACLR},{Freq},{ChBW}'
            OFile.write(OutStr)

##########################################################
### Cleanup Automation
##########################################################
OFile.write("\n")
VSA.jav_Close()



"""Digital Storage Oscilloscope Spectrum Analysis Methods"""
from rssd.DSO.Common import DSO

class DSO(DSO):
    """ Rohde & Schwarz Digital Storage Oscilloscope Object """
    def __init__(self):
        super(DSO, self).__init__()

    #####################################################################
    ### DSO Get Functions
    #####################################################################
    def Get_SA_AcqTime(self):
        """ Seconds """
        rdStr = self.query(':TIM:RANG?')
        return rdStr

    def Get_SA_Resolution_BW(self,Ch):
        """  1 to 8 """
        rdStr = self.query(f':CALC:MATH{Ch}:FFT:BAND:RES:ADJ?')
        return rdStr

    def Get_SA_Frame_Coverage(self,Ch):
        """  1 to 8 """
        rdStr = self.query(f':CALC:MATH{Ch}:FFT:FRAM:COV?')
        return rdStr

    def Get_SA_Timebase_Acquisition_Time(self,Ch):
        """  1 to 8 """
        rdStr = self.query(f':CALC:MATH{Ch}:FFT:FRAM:COV?')
        return rdStr

    #####################################################################
    ### DSO Init Functions
    #####################################################################
    def Init_SpecAn(self):
        #Configure instrument measurment
        pass

    #####################################################################
    ### DSO Set FFT Functions
    #####################################################################
    def Set_SA_AcqTime(self, sec):
        """ Seconds """
        self.write(f':TIM:RANG {sec}')

    def Set_SA_RBW_Auto(self,state,Ch):
        """  ON OFF, Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:BAND:RES:AUTO {state}')

    def Set_SA_RBW_Ratio(self,ratio,Ch):
        """  1 TO 1000 , Ch 1 TO 4"""
        self.write(f':CALC:MATH{Ch}:FFT:BAND:RES:RAT {ratio}')

    def Set_SA_RBW_Value(self,Hz,Ch):
        """ 0.01 to 160e6 Hz , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:BAND:RES:VAL {Hz}')

    def Set_SA_Windowing_Type(self, filtr, Ch):
        """ RECT HAMM HANN BLACKharris GAUSian FLATTOP2
         KAISerbessel  , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:WIND:TYPE {filtr}')

    def Set_SA_Arithmetics(self, func, Ch):
        """ OFF ENV AVER RMS MINH MAXH , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:FRAM:ARIT {func}')

    def Set_SA_Frame_Max(self,count, Ch):
        """ 1 to 10000 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:FRAM:MAXC {count}')
        
    def Set_SA_Frame_Overlap(self,factor, Ch):
        """ 0 TO 90 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:FRAM:OFAC {factor}')

    def Set_SA_Gate_Start_Absolute(self,time, Ch):
        """ -100E+24 to 100e+24 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:ABS:STAR {time}')

    def Set_SA_Gate_Stop_Absolute(self,time, Ch):
        """ -100E+24 to 100e+24 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:ABS:STOP {time}')

    def Set_SA_Gate_Start_Relative(self,percent, Ch):
        """ 0 to 100 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:REL:STAR {percent}')

    def Set_SA_Gate_Stop_Relative(self,percent, Ch):
        """ 0 to 100 , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:REL:STOP {percent}')
    
    def Set_SA_Gate_Mode(self,mode, Ch):
        """ ABS REL , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:ABS:STAR {mode}')  

    def Set_SA_Gate_State(self,state, Ch):
        """ ON OFF , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:GATE:STAT {state}') 

    def Set_SA_Reference_Level(self,dB,Ch):
        """ -1E15 to 1E15 dB , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:MAGN:LEV {dB}')

    #def Set_SA_Vertical_Range(self,dB, Ch):#NOT WORKING
    #    """ 1 to 500 dB , Ch 1 TO 4 """
    #    self.write(f':CALC:MATH{Ch}:FFT:MAGN:RANGE {dB}') 

    def Set_SA_Math(self,state,Ch):
        """  ON OFF , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:STAT {state}')

    def Set_SA_Center_Freq(self,Hz, Ch):
        """ 0 to 2E12 Hz, Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:CFR {Hz}')

    def Set_SA_Full_Span(self,Ch):
        """ 0 to 2E12 Hz , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:FULL')

    def Set_SA_Span(self,Hz,Ch):
        """ 1 to 4E12 Hz , Ch 1 TO 4 """
        self.write(f':CALC:MATH{Ch}:FFT:SPAN {Hz}')

    #def Set_SA_Vertical_Scale(self,unit,Ch):#CNOT WORKING
    #    """ LIN DBM DB DBUV DBV , Ch 1 TO 4 """
    #    self.write(f':CALC:MATH{Ch}:FFT:MAGN:SCAL {unit}') 

###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    DSO_Inst = DSO().jav_Open("192.168.1.100")
    DSO_Inst.jav_IDN()
    DSO_Inst.jav_ClrErr()



"""Instrument IDN String"""
from rssd.instrument import instr

for IPAddr in ['192.168.58.109','192.168.58.114','127.0.0.1']:
    kyber = instr().open(IPAddr)
    print(kyber.query('*IDN?'))
    kyber.close()



"""rssd.VSA.ADemod_K7 test
"""
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.Transient_K60     import VSA          #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_TranAna()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_TransientAnalysis_Chirp_Get(self):
        self.FSW.Set_TA_Mode('CHIRP')
        self.FSW.Set_SweepCont(0)
        self.FSW.jav_Wait('INIT:IMM')
        self.FSW.Get_TA_ChirpTable()
        self.FSW.Get_TA_ChirpStat()
        self.FSW.Get_TA_ChirpStats_TimeBegin()
        self.FSW.Get_TA_ChirpStats_TimeLength()
        self.FSW.Get_TA_ChirpStats_Rate()
        self.FSW.Get_TA_ChirpStats_StateDev()
        self.FSW.Get_TA_ChirpStats_AvgFreq()
        self.FSW.Get_TA_ChirpStats_Bandwidth()
        self.FSW.Get_TA_ChirpStats_FreqDevAvg()

    def test_FSW_TransientAnalysis_Hop_Get(self):
        self.FSW.Set_TA_Mode('HOP')
        self.FSW.Set_SweepCont(0)
        self.FSW.jav_Wait('INIT:IMM')
        self.FSW.Get_TA_HopTable()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
### Purpose: rssd.RCT.Common test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.160'              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.RCT.Common    import RCT

class TestGeneral(unittest.TestCase):
    def setUp(self):                      #run before each test
        self.CMP = RCT().jav_OpenTest(host)

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.CMP.jav_Error()[0],'0')
        self.CMP.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_RCT_Ex_GenArb(self):
        self.CMP.Init_Gen()
        self.CMP.Set_Gen_Port('P1.IFOut')
        self.CMP.Set_Gen_Port_State('ON')
        self.CMP.Set_Gen_Freq(6e9)
        self.CMP.Set_Gen_RFPwr(-50)
        self.CMP.Set_Gen_Mode('ARB')
        self.CMP.Set_Gen_ArbWv('Test.wv')
        self.CMP.Set_Gen_ArbExec()
        self.CMP.Set_Gen_RFState('ON')

    def test_RCT_Ex_GenCW(self):
        self.CMP.Init_Gen()
        self.CMP.Set_Gen_Port('P1.IFOut')
        self.CMP.Set_Gen_Port_State('ON')
        self.CMP.Set_Gen_Freq(6e9)
        self.CMP.Set_Gen_RFPwr(-50)
        self.CMP.Set_Gen_Mode('CW')
        self.CMP.Set_Gen_RFState('ON')

    def test_RCT_Ex_GenListMode(self):
        self.CMP.Init_Gen()
        self.CMP.Set_Gen_Port('P1.IFOut')
        self.CMP.Set_Gen_Port_State('ON')
        self.CMP.Set_Gen_Freq(6e9)
        self.CMP.Set_Gen_RFPwr(-50)
        self.CMP.Set_Gen_ListMode('ON')
        self.CMP.Set_Gen_RFState('ON')
        self.CMP.Set_Gen_ListMode('OFF')

    def test_RCT_Ex_MeasFFT(self):
        self.CMP.Init_Meas_FFT()
        self.CMP.Set_Meas_Freq(6e9)
        self.CMP.Set_Meas_Port('P1.IFIn')
        self.CMP.Set_Meas_Autolevel()
        # self.CMP.Set_Meas_RefLevl(-10)
        # self.CMP.Set_Meas_UserMargin(0)
        self.CMP.Set_Meas_RFBW(10e6)
        self.CMP.Set_Meas_Span(100e6)

    def test_RCT_Ex_MeasPwr(self):
        self.CMP.Init_Meas_Power()
        self.CMP.Set_Meas_Freq(6e9)
        self.CMP.Set_Meas_UserMargin(0)
        self.CMP.Set_Meas_Expected_Nom_Power(0)
        self.CMP.Set_Meas_TriggerSource('IF Power')
        self.CMP.Set_Meas_TriggerThreshold(-10)
        self.CMP.Set_Meas_TriggerThreshold(-40)
        self.CMP.Set_Meas_Pwr_MLength(100e-6)
        self.CMP.Set_Meas_RFBW(100e6)
        self.CMP.Get_Meas_Power()

    def test_RCT_Get_Gen(self):
        self.CMP.Get_Gen_ArbWv()
        self.CMP.Get_Gen_Freq()
        self.CMP.Get_Gen_Mode()
        self.CMP.Get_Gen_Port()

    def test_RCT_Get_Meas(self):
        self.CMP.Get_Meas_Port()

    def test_RCT_System(self):
        self.CMP.Get_Options()
        self.CMP.Init_Syst()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""
### Purpose : NRP Power Sensor
###
### VISAFmt : RSNRP::0x0138::100961::INSTR
###           RSNRP::<Modl>::<SerN>::INSTR
###           USB::0x0AAD::<Modl>::<SerN>::INSTR
###
### Product  |USB ID      Product  |USB ID
### ---------|------      ---------|------
### NRP8S     0x00E2      NRP33SN-V 0x0168
### NRP8SN    0x0137      NRP40S    0x015F
### NRP18S    0x0138      NRP40SN   0x0160
### NRP18SN   0x0139      NRP50S    0x0161
### NRP33S    0x0145      NRP50SN   0x0162
### NRP33SN   0x0146      NRPM      0x0195
###
### Product  |USB ID      Product  |USB ID      Product  |USB ID
### ---------|------      ---------|------      ---------|------
### NRP-Z11  0x00C        NRP-Z31    0x02C      NRP-Z81    0x023
### NRP-21   0x003        NRP-Z37    0x02D      NRP-Z85    0x083
### NRP-Z22  0x013        NRP-Z51    0x016      NRP-Z86    0x095
### NRP-Z23  0x014        NRP-Z52    0x017      NRP-Z91    0x021
### NRP-Z24  0x015        NRP-Z55    0x018      NRP-Z92    0x062
### NRP-Z27  0x02F        NRP-Z56    0x019      NRP-Z98    0x052
### NRP-Z28  0x051        NRP-Z57    0x070"""
import numpy as np
from rssd.yaVISA import jaVisa

class PMr(jaVisa):
    """ Rohde & Schwarz Power Meter Object """
    def __init__(self):
        super(PMr,self).__init__()     # Python2/3
        self.Model = "NRP"

    #####################################################################
    ### NRP Get Methods
    #####################################################################
    def Get_AvailableNRP(self):
        resList = self.jav_reslist()
        asdf = [s for s in resList if "USB0::0x0AAD::0x" in s]
        print(asdf)

    def Get_Average(self):
        outp = self.queryInt('SENS:AVER:COUN?')
        return outp

    def Get_Freq(self):
        outp = self.query(':SENS:FREQ?\n')
        return outp

    def Get_BufferedMeas(self,bState):
        if (bState == 1) or (bState == 'ON'):
            self.write('SENS:POW:AVG:BUFF:STAT ON')                #Configure a buffered measurement
        else:
            self.write('SENS:POW:AVG:BUFF:STAT ON')                #Configure a buffered measurement

    def Get_EventStatus(self):
        outp = self.query('STAT:OPER:MEAS:EVEN?')
        return outp

    def Get_Offset(self):
        ### Offset = Loss
        ### +Num => +Reading
        ### -Num ==> -Reading
        outp = self.queryFloat('SENS:CORR:OFFS?')
        return outp

    def Get_Power(self):
        # self.write('UNIT:POW DBM')  # Not a function for NRP-Zxx
        # self.write('SENS:FUNC "POW:AVG"')
        self.write(':INIT:IMM')
        outp = self.query('FETCH?\n')
        self.write(':INIT:CONT OFF')
        print('Done')
        x = np.fromstring(outp,dtype=np.float, sep=',')             # Convert String to Float
        y = np.multiply(10,(np.log10(abs(np.multiply(x,1000)))))    # Convert Watts to dBm
        return outp

    def Get_NRPM_PowerAll(self):
        ### NRP3M
        self.write('UNIT:POW DBM')
        # self.write('SENS:FUNC "POW:AVG"')
        self.write('SENS:CHAN1:ENAB ON')
        self.write('SENS:CHAN2:ENAB ON')
        self.write('SENS:CHAN3:ENAB ON')
        self.query('INIT:IMM;*OPC?')
        outp = self.queryFloat('FETCH:ALL?')
        return outp

#####################################################################
### NRP Set Methods
#####################################################################
    def Set_Aperture(self,fAPR):
        self.write('SENS:POW:AVG:APER %f'%fAPR)

    def Set_Average(self,iAvg):
        self.write('SENS:AVER:COUN %d'%iAvg)

    def Set_AverageMode(self,sState):
        """ON | OFF"""
        if sState in (1,'1','ON'):
            self.write('SENS:AVER:COUN:AUTO OFF')
        elif sState in (0,'0','OFF'):
            self.write('SENS:AVER:COUN:AUTO ON')

    def Set_BufferSize(self,iSize):
        self.write('SENS:POW:AVG:BUFF:SIZE %d'%iSize)                #Buffer size is randomly selected to 17

    def Set_Freq(self,fFreq):
        self.write('SENS:FREQ %f'%fFreq)

    def Set_Function(self,sFunc):
        """ POW:AVG """
        self.write('SENS:FUNC "%s"'%sFunc)

    def Set_InitImm(self):
        self.write('INIT:IMM')

    def Set_InitCont(self,sState):
        """ ON; OFF """
        self.write('INIT:CONT %s'%sState)

    #####################################################################
    ### NRPM
    ###    - NRP-ZKU    USB cable (3.0m) to R&S®NRPxxS(N)
    ###    - NRPM3      OTA power sensor
    ###    - NRPM-ZKD3 Interface cable; R&S®NRPM3 to R&S®NRPM-ZD3
    ###    - NRPM-ZD3  Cable feedthrough for anechoic chamber
    ###    - NRPM-Axx  OTA Antenna module: A66(27-75)
    #####################################################################
    def Set_NRPM_LED(self,sState,iSensor=1):
        """ON | OFF"""
        if sState in (1,'1','ON'):
            self.write('SYST:LED:CHAN%d:COL 255'%iSensor)
        elif sState in (0,'0','OFF'):
            self.write('SYST:LED:CHAN%d:COL 0'%iSensor)

    def Set_PowerOffset(self,fOffset):
        self.write('SENS:CORR:OFFS:STAT ON')
        self.write('SENS:CORR:OFFS %f'%fOffset)

    def Set_PowerOffsetState(self,sState):
        """ON | OFF"""
        if sState in (1,'1','ON'):
            self.write('SENS:CORR:OFFS:STAT ON')
        elif sState in (0,'0','OFF'):
            self.write('SENS:CORR:OFFS:STAT OFF')

    def Set_TriggerSource(self,sSource):
        """BUS; EXT2; INT; IMM """
        self.write('TRIG:SOUR %s'%sSource)

    def Set_TriggerAuto(self,sState):
        """ON | OFF"""
        if sState in (1,'1','ON'):
            self.write('TRIG:ATR:STAT ON')          #Auto-Trigger ON
        elif sState in (0,'0','OFF'):
            self.write('TRIG:ATR:STAT OFF')         #Auto-Trigger OFF

    def Set_TriggerCount(self,iNum):
        self.write('TRIG:COUN %d'%iNum)


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    # this won't be run when imported
    NRP = PMr()
    NRP.Get_AvailableNRP()
    NRP.jav_openvisa("USB0::0x0AAD::0x0196::900105::INSTR")
    NRP.jav_logscpi()
    NRP.Set_Freq(24e9)
    NRP.Get_Power()
    NRP.jav_ClrErr()



"""OSP Open Switch Platform Common Functions"""
from rssd.yaVISA import jaVisa

class OSP(jaVisa):
    """ Rohde & Schwarz Open Switch Platform Object """
    def __init__(self):
        super(OSP, self).__init__()
        self.Model = "OSP1x0"

    #####################################################################
    ### OSP Switching Functions
    #####################################################################
    def Get_OSP_Info(self):
        return self.query('DIAG:SERV:HWIN?').split(',')

    def Get_OSP_Modules(self):
        rdstr = self.query('ROUT:MOD:CAT?')
        return rdstr

    def Get_SW_SPDT(self,slot=11,sw=1):
        """ Slot, Switch """
        # ROUT:CLOS? (@F01A11(0161))
        outstr = f'ROUT:CLOS? (@F01A{slot:02d}(01{sw:02d}))'
        print(outstr)
        state = self.queryInt(outstr)
        print("A%02d SW%d @Pos%d"%(slot,sw,state))
        return int(state)

    def Get_SW_SP6T(self,slot=11,sw=1):
        """ Slot, Switch """
        # ROUT:CLOS? (@F01A11(0161))
        CurrState = 1000                    # default test value
        for pos in range(0,7):
            state = self.queryInt(f'ROUT:CLOS? (@F01A{slot:02d}({pos:02d}{sw:02d}))')
            if state == 1:
                CurrState = pos
                print(f"A{slot:02d} SW{sw:02d} @Pos{pos:02d}")
        return CurrState

    def Set_CompatabilityMode(self,sState):
        """
            F01Mxx --> F01A1x nomenclature
            F01M01 --> F01A11
            ON OFF 1 0
        """
        if sState in (1,'1','ON'):
            self.write(f'CONF:COMP ON')
        elif sState in (0,'0','OFF'):
            self.write(f'CONF:COMP OFF')

    def Set_SW(self,slot=11,sw=1,pos=1):
        # ROUT:CLOS (@F01A11(0161))
        outstr = f'ROUT:CLOS (@F01A{slot:02d}({pos:02d}{sw:02d}))'
        print(outstr)
        self.write(outstr)

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    RFU3 = OSP()
    RFU3.jav_openvisa('TCPIP0::192.168.1.150::INSTR')
    RFU3.Set_CompatabilityMode(1)
    print(RFU3.Get_OSP_Info())
    print(RFU3.Get_SW_SPDT(11,12))
    RFU3.Set_SW(11,12,0)
    # RFU3.query('ROUT:CLOS? (@F01A11(0111))')
    RFU3.jav_ClrErr()



"""abstract Rohde & Schwarz Communication Bus Class"""
from abc import ABC, abstractmethod

class bus(ABC):
    """abstract Rohde & Schwarz Communication Bus Class"""

    @abstractmethod
    def close(self): pass

    @abstractmethod
    def open(self, resourceID, param=None): pass

    @abstractmethod
    def query(self, SCPIstr): pass

    @abstractmethod
    def read_raw(self): pass

    @abstractmethod
    def timeout(self, seconds): pass

    @abstractmethod
    def write(self, SCPIstr): pass

    @abstractmethod
    def write_raw(self, SCPIstr): pass



""" Rohde & Scharz Instrument Functions"""
import feedparser

def parse_feed_firmware():
    url = 'https://www.rohde-schwarz.com/us/rss-feeds/firmware-feed_229511.rss'
    feed = feedparser.parse(url)
    print(f'\n{url}')
    for entry in feed.entries:
        entry_title     = entry.title.replace('Firmware for R&S®','')
        entry_link      = entry.link
        entry_published = entry.published           # Unicode string
        # entry_parsed    = entry.published_parsed    # Time object
        # entry_content   = entry.summary

        print(f"{entry_title[0:30]:32}[{entry_link:55}] {entry_published}")

def parse_feed_drivers():
    url = 'https://www.rohde-schwarz.com/us/rss-feeds/driver-feed_229510.rss'
    feed = feedparser.parse(url)
    print(f'\n{url}')
    for entry in feed.entries:
        entry_title     = entry.title.replace('Drivers for R&S®','')
        entry_link      = entry.link
        entry_published = entry.published           # Unicode string

        print(f"{entry_title[0:30]:32}[{entry_link:55}] {entry_published}")

def parse_feed_appNote():
    url = 'https://www.rohde-schwarz.com/us/rss-feeds/application-note-feed_229508.rss'
    feed = feedparser.parse(url)
    print(f'\n{url}')
    for entry in feed.entries:
        entry_title     = entry.title.replace('R&S®','')
        entry_link      = entry.link.replace('https://www.rohde-schwarz.com/us/applications','<RSA-Apps>')

        print(f"{entry_title[0:40]:42}[{entry_link:55}]")

###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    parse_feed_drivers()
#     parse_feed_firmware()
    parse_feed_appNote()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
#pylint: disable=E0611,E0401
###############################################################################
FSW_IP      = '192.168.58.109'

###############################################################################
### Overhead
###############################################################################
from rssd.VSA.NR5G_K144     import VSA

FSW = VSA().jav_Open(FSW_IP)                          #Create FSW Object
FSW.debug = 0

###############################################################################
### Code Start
###############################################################################
FSW.Set_Freq(3.5e9)
FSW.Init_5GNR()
FSW.Set_5GNR_FrameCount('OFF')
FSW.Set_5GNR_Direction('UL')
# FSW.Set_5GNR_CellID(1024)
FSW.Set_5GNR_FreqRange('LOW')
FSW.Set_5GNR_ChannelBW(100)
FSW.write(f'CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:ALC 0')                               # PUSCH Allocations
FSW.write(f'CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:UCCC 50')                             # PUCCH Allocation
FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:CSL 1')                                    # User configurable slots, rest copy

for ch in range(0,44):
    ## Frame Configuration
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:FORM 0')                # Format
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:RBC 1')                 # Number of RB
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:RBOF {ch}')             # RB Offset
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:SCO 2')                 # Number of Symbol
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:SOFF 12')               # Symbol Offset
    # FSW.write(f':SOUR1:BB:NR5G:SCH:CELL0:SUBF0:USER{ch}:BWP0:ALL0:REP SLOT')      # Repitition

    ## PUCCH Setting
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:ISFH ON')          # Intra Slot Freq Hop
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:HID NID')          # Hop ID type
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:NID {ch * 2}')     # Hop ID num
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:SHPR {272-ch}')    # 2nd Hop PBR
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:ICSH 6')           # Initial Cyclic Shift

for ch in range(44,50):
    ## Frame Configuration
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:FORM 2')                # Format
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:RBC 1')                 # Number of RB
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:RBOF {ch}')             # RB Offset
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:SCO 2')                 # Number of Symbol
    FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:SOFF 0')                # Symbol Offset

    ## PUCCH Setting
    # FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:ISFH OFF')          # Intra Slot Freq Hop
    # FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:HID NID')          # Hop ID type
    # FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:NID {ch * 2}')     # Hop ID num
    # FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:SHPR {272-ch}')    # 2nd Hop PBR
    # FSW.write(f':CONF:NR5G:UL:CC1:FRAM1:BWP0:SLOT0:PUCC{ch}:DMRS:ICSH 6')           # Initial Cyclic Shift



################################################################################
### Rohde & Schwarz Automation for demonstration use.
###
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date   : 2018.05.24
################################################################################
### User Entry
################################################################################
VSA_IP  = '192.168.1.109'
VSG_IP  = '192.168.1.114'
MeasTim = 0
Freq    = 9.0e9
RBW     = 200e3
ChBW    = 90e6
ChSpace = 100e6
DUTGain = 50
Avg     = 0
SweMode = 'AUTO'            #AUTO | SPEed | DYN
SweType = 'AUTO'            #AUTO | SWE | FFT

### Loops
Repeat  = 1
PwrSweep = 59

################################################################################
### Code Overhead
################################################################################
import timeit
from rssd.VSA.Common        import VSA              #pylint: disable=E0611,E0401
from rssd.yaVISA_socket     import jaVisa           #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO           #pylint: disable=E0611,E0401
# import rssd.VSA_Leveling    as VSAL               #pylint: disable=E0611,E0401

OFile = FileIO().makeFile(__file__)
VSA = VSA().jav_Open(VSA_IP,OFile)                  #Create VSA Object
VSG = jaVisa().jav_Open(VSG_IP,OFile)               #Create Object
################################################################################
### Code Start
################################################################################
# VSA.jav_Reset()
VSA.Set_Freq(Freq)
VSA.Set_Param_Couple_All()
VSA.Init_ACLR()                                     #VSA ACLR Channel
VSA.Set_ACLR_CHBW(ChBW)
VSA.Set_ACLR_AdjBW(ChBW)
VSA.Set_ACLR_AdjSpace(ChSpace)
VSA.Set_ACLR_NumAdj(1)

VSA.Set_ResBW(RBW)
VSA.Set_SweepTime(MeasTim)
VSA.Set_Trace_Avg('LIN')
VSA.Set_Trace_AvgCount(Avg)
VSA.Set_Trace_Detector('RMS')
VSA.Set_SweepOpt(SweMode)
VSA.Set_SweepType(SweType)
VSA.Set_In_YIG('OFF')
# VSA.Set_Trig1_Source('Ext')

################################################################################
### Measure Time
################################################################################
#sDate = timeit.default_timer().strftime("%y%m%d-%H:%M:%S.%f") #Date String
LoopParam   = 'Iter,Pwr'
TimeParam   = 'AlTime,MeasTime,TotalTIme'
SwpParam    = VSA.Get_Params_Sweep(1)
AmpParam    = VSA.Get_Params_Amp(1)
TrcParam    = VSA.Get_Params_Trace(1)
SysParam    = VSA.Get_Params_System(1)
MeasData    = 'TxPwr,Adj-,Adj+,Alt-,Alt+'
OFile.write(f'{LoopParam},{TimeParam},{AmpParam},{SwpParam},{TrcParam},{SysParam},{MeasData}')
# table = VSAL.ReadLevellingTables(Freq)

tick0 = timeit.default_timer()
for i in range(Repeat):
    for VSApwr in range(PwrSweep):
        tick = timeit.default_timer()

        ### <\thing we are timing>
        VSG.write(f':POW:AMPL {VSApwr - DUTGain}dbm')           # VSG Power
        #################
        ### AUTOLEVEL ###
        #################
        VSA.Set_SweepCont(0)                                # Sweep Continuous
        VSA.Set_Autolevel()
        # else:
        #     lvlTable = VSA.Set_Autolevel_IQIF(table)
        tockA =  timeit.default_timer()
        VSA.Set_Channel('SAN')
        VSA.Set_SweepCont(0)                                    # Single Sweep
        VSA.Set_InitImm()                                       # Take Sweep
        ACLR = VSA.query(':CALC:MARK:FUNC:POW:RES? MCAC')
        ### <\thing we are timing>
        tockB    = timeit.default_timer()
        ALTime   = (tockA-tick)
        TotTime  = (tockB-tick)
        TestTime = TotTime - ALTime
        # OutStr   = f'{i},{RBW},{MeasTim},{-50+pwr},{ALTime},{TotTime},{AmpSet},{SwpParam},{ACLR}'

        ##############################################################
        ### LOG DATA
        ##############################################################
        LoopParam   = f'{i},{VSApwr:5.2f}'
        SwpParam    = VSA.Get_Params_Sweep()
        AmpParam    = VSA.Get_Params_Amp()
        TrcParam    = VSA.Get_Params_Trace()
        SysParam    = VSA.Get_Params_System()
        TotTime     = f'{ALTime:2,.6f},{TestTime:2,.6f},{TotTime:2,.6f}'
        MeasData    = ACLR
        OutStr      = f'{LoopParam},{TotTime},{AmpParam},{SwpParam},{TrcParam},{SysParam},{MeasData}'
        OFile.write(OutStr)

# VSAL.WriteLevellingTables(Freq, table)
SuiteTime = timeit.default_timer() - tick0
print(f'Total Test time : {SuiteTime:2,.6f}')
print(f'Time/Measurement: {SuiteTime/(Repeat*PwrSweep):2,.6f}')

################################################################################
### Cleanup Automation
################################################################################
OFile.write("\n")
VSA.jav_Close()



"""FSW/SMW LTE Demo"""
#pylint: disable=E0611,E0401
from rssd.VSG.WLAN_K54  import VSG
from rssd.VSA.WLAN_K91  import VSA

class VST(object):
    """ Rohde & Schwarz Vector Signal Transciever 802.11 Object """
    def __init__(self):
        self.Freq      = 2.5e9
        self.SWM_Out   = 0
        self.WLAN_Std  = 'N'
        self.WLAN_ChBW = 20        #MHz
        self.WLAN_MCS  = 1
        self.WLAN_Mod  = 'QAM64'    #QPSK; QAM16; QAM64; QAM256

    def Get_WLAN_All(self):
        odata =  [[] for i in range(3)]
        odata[0].append("[[Parameter]]  ")
        odata[0].append("Standard       ")
        odata[0].append("Ch BW          ")
        odata[0].append("PPDU           ")
        odata[0].append("MCS            ")
        odata[0].append("Mod            ")

        try:
            odata[1].append("[-SMW-]")
            odata[1].append(self.SMW.Get_WLAN_Standard())
            odata[1].append(self.SMW.Get_WLAN_ChBW())
            odata[1].append(self.SMW.Get_WLAN_PPDU())
            odata[1].append(self.SMW.Get_WLAN_MCS())
            odata[1].append(self.SMW.Get_WLAN_Modulation())
        except:                                                                 #pragma: no cover
            pass

        try:
            self.FSW.Init_WLAN()
            odata[2].append("[-FSW-]")
            odata[2].append(self.FSW.Get_WLAN_Standard())
            odata[2].append(self.FSW.Get_WLAN_ChBW())
            odata[2].append(self.FSW.Get_WLAN_PPDU())
            odata[2].append(self.FSW.Get_WLAN_MCS())
            odata[2].append(self.FSW.Get_WLAN_Modulation())
        except:                                                                 #pragma: no cover
            pass
        print('SMW/FSW Values: %d %d'%(len(odata[2]),len(odata[2])))

        return odata

    def Get_WLAN_All_print(self):
        data = self.Get_WLAN_All()
        for i in range(len(data[0])):
            try:
                print("%s\t%s\t%s"%(data[0][i],data[1][i],data[2][i]))
            except:                                                             #pragma: no cover
                try:
                    print("%s\t%s\t%s"%(data[0][i],data[1][i],'<notRead>'))
                except:
                    print("%s\t%s\t%s"%(data[0][i],'<notRead>',data[2][i]))

    def jav_Open(self,SMW_IP,FSW_IP,OFile=''):
        self.SMW = VSG().jav_Open(SMW_IP,OFile,prnt=0)  #Create SMW Object
        self.FSW = VSA().jav_Open(FSW_IP,OFile,prnt=0)  #Create FSW Object
        return self

    def jav_OpenTest(self,SMW_IP,FSW_IP):
        self.SMW = VSG().jav_OpenTest(SMW_IP)  #Create SMW Object
        self.FSW = VSA().jav_OpenTest(FSW_IP)  #Create FSW Object
        return self

    def jav_Close(self):
        self.SMW.jav_Close()
        self.FSW.jav_Close()

    def jav_Clear(self):
        self.SMW.jav_Clear()
        self.FSW.jav_Clear()

    def Set_WLAN_All(self):
        try:
            ### SMW Settings
            self.SMW.Set_Freq(self.Freq)
            self.SMW.Set_WLAN_BBState('OFF')
            self.SMW.Set_WLAN_ChBW(self.WLAN_ChBW)
            self.SMW.Set_WLAN_Standard(self.WLAN_Std)
            self.SMW.Set_WLAN_MCS(self.WLAN_MCS)
            self.SMW.Set_WLAN_BBState('ON')
            self.SMW.Set_RFState('ON')                                          #Turn RF Output on
        except:                                                                 #pragma: no cover
            print("WLAN_SetSettings: SMW Error")

        try:
            ### FSW Setting
            self.FSW.Init_WLAN()
            self.FSW.Set_Freq(self.Freq)
            self.FSW.Set_WLAN_Standard(self.WLAN_Std)
            self.FSW.Set_WLAN_ChBW(self.WLAN_ChBW)
            self.FSW.Set_WLAN_MCS(self.WLAN_MCS)
        except:                                                                 #pragma: no cover
            print("WLAN_SetSettings: FSW Error")
        return 0


##########################################################
### Instrument Settings
##########################################################
if __name__ == "__main__":
    WLAN = VST().jav_Open('192.168.1.114','192.168.1.109')
    WLAN.WLAN_Std  = 'AC'
    WLAN.WLAN_ChBW = 160
    WLAN.WLAN_MCS  = 5
    WLAN.Set_WLAN_All()
    WLAN.Get_WLAN_All_print()
    WLAN.jav_Close()
